<?php
/**
* 2007-2017 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author PrestaShop SA <contact@prestashop.com>
*  @copyright  2007-2017 PrestaShop SA
*  @license    http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/

if (!defined('_PS_VERSION_')) {
    exit;
}
use PrestaShop\PrestaShop\Core\Module\WidgetInterface;
use PrestaShop\PrestaShop\Core\Product\ProductExtraContent;
use PrestaShop\PrestaShop\Adapter\Image\ImageRetriever;
use PrestaShop\PrestaShop\Adapter\Product\PriceFormatter;
use PrestaShop\PrestaShop\Core\Product\ProductListingPresenter;
use PrestaShop\PrestaShop\Adapter\Product\ProductColorsRetriever;

require(dirname(__FILE__).'/classes/StProductCommentClass.php');
require(dirname(__FILE__).'/classes/StProductCommentCriterionClass.php');

class StProductComments extends Module implements WidgetInterface
{
    public $_html = '';
    protected static $access_rights = 0775;
    protected $secure_key;
    public $fields_form;
    public $fields_value;
    public $validation_errors = array();
    public $_prefix_st = 'ST_PROD_C_';
    public $_prefix_stsn = 'STSN_PROD_C_';
    public $_hooks = array();
    public $_pages = array();
    public $text_width = array();
    public $wide_map = array(
        array('id'=>'1', 'name'=>'1/12'),
        array('id'=>'1-2', 'name'=>'1.2/12'),
        array('id'=>'1-5', 'name'=>'1.5/12'),
        array('id'=>'2', 'name'=>'2/12'),
        array('id'=>'2-4', 'name'=>'2.4/12'),
        array('id'=>'3', 'name'=>'3/12'),
        array('id'=>'4', 'name'=>'4/12'),
        array('id'=>'5', 'name'=>'5/12'),
        array('id'=>'6', 'name'=>'6/12'),
        array('id'=>'7', 'name'=>'7/12'),
        array('id'=>'8', 'name'=>'8/12'),
        array('id'=>'9', 'name'=>'9/12'),
        array('id'=>'10', 'name'=>'10/12'),
        array('id'=>'11', 'name'=>'11/12'),
        array('id'=>'12', 'name'=>'12/12'),
    );
    public $sort_order = array();
    public $order_states = array();
    public $lang_field = array('extra_message', 'email_review_subject', 'email_reply_subject', 'email_reminder_subject');
    protected $fields_default_stsn = array(
        'pro_per_fw' => 0,
        'pro_per_xxl' => 1,
        'pro_per_xl' => 1,
        'pro_per_lg' => 1,
        'pro_per_md' => 1,
        'pro_per_sm' => 1,
        'pro_per_xs' => 1,
    );
    private $_st_themes = false;
    public function __construct()
    {
        $this->name          = 'stproductcomments';
        $this->tab           = 'front_office_features';
        $this->version       = '2.0.1';
        $this->author        = 'ST-themes';
        $this->need_instance = 0;
        $this->bootstrap     = true;
        $this->module_key     = 'ca858bfaf56e7edece0f39074f0d89c7';
        
        $this->secure_key = Tools::encrypt($this->name);
        parent::__construct();
        
        $this->displayName = $this->l('PrestaShop Product Reviews module.');
        $this->description = $this->l('Ultimate 4 in 1 Product ratings & Product reviews + Google customer reviews + Google rich snippets + Google reCAPTCHA.');
        $this->ps_versions_compliancy = array('min' => '1.7', 'max' => _PS_VERSION_);
        $this->controllers = array('list','detail');

        $this->sort_order = array(
            array('id'=>0, 'name'=>$this->l('Popularity')),
            array('id'=>1, 'name'=>$this->l('Most helpful')),
            array('id'=>2, 'name'=>$this->l('Newest')),
            array('id'=>3, 'name'=>$this->l('Oldest')),
            array('id'=>4, 'name'=>$this->l('Most commented')),
        );
        
        $this->text_width = array(
            array('id' => 90, 'name'=> $this->l('Left').' 90%'),
            array('id' => 91, 'name'=> $this->l('Center').' 90%'),
            array('id' => 92, 'name'=> $this->l('Right').' 90%'),
            array('id' => 80, 'name'=> $this->l('Left').' 80%'),
            array('id' => 81, 'name'=> $this->l('Center').' 80%'),
            array('id' => 82, 'name'=> $this->l('Right').' 80%'),
            array('id' => 70, 'name'=> $this->l('Left').' 70%'),
            array('id' => 71, 'name'=> $this->l('Center').' 70%'),
            array('id' => 72, 'name'=> $this->l('Right').' 70%'),
            array('id' => 60, 'name'=> $this->l('Left').' 60%'),
            array('id' => 61, 'name'=> $this->l('Center').' 60%'),
            array('id' => 62, 'name'=> $this->l('Right').' 60%'),
            array('id' => 50, 'name'=> $this->l('Left').' 50%'),
            array('id' => 51, 'name'=> $this->l('Center').' 50%'),
            array('id' => 52, 'name'=> $this->l('Right').' 50%'),
            array('id' => 30, 'name'=> $this->l('Left').' 30%'),
            array('id' => 31, 'name'=> $this->l('Center').' 30%'),
            array('id' => 32, 'name'=> $this->l('Right').' 30%'),
            array('id' => 20, 'name'=> $this->l('Left').' 20%'),
            array('id' => 21, 'name'=> $this->l('Center').' 20%'),
            array('id' => 22, 'name'=> $this->l('Right').' 20%'),
            array('id' => 10, 'name'=> $this->l('Left').' 10%'),
            array('id' => 11, 'name'=> $this->l('Center').' 10%'),
            array('id' => 12, 'name'=> $this->l('Right').' 10%'),
        );

        $this->_pages = array(
            array(
                'id' => 'index',
                'val' => 1,
                'name' => $this->l('Homepage')
            ),
            array(
                'id' => 'category',
                'val' => 1,
                'name' => $this->l('Category')
            ),
            array(
                'id' => 'product',
                'val' => 1,
                'name' => $this->l('Product')
            ),
            array(
                'id' => 'pricesdrop',
                'val' => 1,
                'name' => $this->l('Prices Drop')
            ),
            array(
                'id' => 'newproducts',
                'val' => 1,
                'name' => $this->l('New Products')
            ),
            array(
                'id' => 'manufacturer',
                'val' => 1,
                'name' => $this->l('Manufacturer')
            ),
            array(
                'id' => 'supplier',
                'val' => 1,
                'name' => $this->l('Supplier')
            ),
            array(
                'id' => 'bestsales',
                'val' => 1,
                'name' => $this->l('Best Sales')
            ),
            array(
                'id' => 'cms',
                'val' => 1,
                'name' => $this->l('Cms')
            ),
        );
        $this->theme_name = preg_replace("/(_?child)$/", '', Tools::strtolower(Context::getContext()->shop->theme->getName()));
        $this->_st_themes = $this->theme_name=='panda' ? 2 : ($this->theme_name=='transformer' ? 1 : false);
    }

    public function install()
    {
        if (!parent::install()
            || !$this->installDB()
            || !$this->registerHook('actionProductDelete')
            || !$this->registerHook('displayProductExtraContent')
            || !$this->registerHook('displayHeader')
            || !$this->registerHook('displayFooter')
            || !$this->registerHook('displayTop')
            || !$this->registerHook('displayOrderConfirmation')
            || !$this->registerHook('displayProductListReviews')
            || !$this->registerHook('displayCustomerAccount')
            || !$this->registerHook('displayProductAdditionalInfo')
            || !$this->registerHook('displayBeforeBodyClosingTag')
            || !$this->registerHook('registerGDPRConsent')
            || !$this->registerHook('actionDeleteGDPRCustomer')
            || !$this->registerHook('actionExportGDPRData')
        ) {
            return false;
        }
        $result = true;
        foreach ($this->getFormFieldsDefault() as $k => $v) {
            $result &= Configuration::updateValue($this->_prefix_st.Tools::strtoupper($k), $v);
        }
        foreach ($this->fields_default_stsn as $k => $v) {
            $result &= Configuration::updateValue($this->_prefix_stsn.Tools::strtoupper($k), $v);
        }
        if($this->_st_themes){
            $result &= Configuration::updateValue($this->_prefix_st.'COMPATIBILITY_MODE', 1);
            $result &= Configuration::updateValue($this->_prefix_st.'LOAD_SWIPER', 1);
            $result &= Configuration::updateValue($this->_prefix_st.'LOAD_MAGNIFIC_POPUP', 1);
        }
        $result &= $this->sampleData();
        return (bool)$result;
    }
    public function installDb()
    {
        $return = true;
        $return &= Db::getInstance()->execute('
			CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'st_product_comment` (
            `id_st_product_comment` int(11) unsigned NOT NULL auto_increment,
            `id_parent` int(10) unsigned NOT NULL DEFAULT 0,
            `id_order_detail` int(10) unsigned NOT NULL DEFAULT 0,
            `id_product` int(10) UNSIGNED NOT NULL DEFAULT 0,
            `id_shop` int(10) UNSIGNED NOT NULL DEFAULT 0,
            `id_customer` int(10) unsigned NOT NULL DEFAULT 0,
            `id_guest` int(10) unsigned DEFAULT NULL DEFAULT 0,
            `email` varchar(64) NULL,
            `title` varchar(64) NULL,
            `content` text NOT NULL,
            `customer_name` varchar(64) DEFAULT NULL,
            `grade` float unsigned NOT NULL,
            `validate` tinyint(1) UNSIGNED DEFAULT 0,
            `deleted` tinyint(1) UNSIGNED DEFAULT 0,
            `is_admin` tinyint(1) UNSIGNED DEFAULT 0,
            `featured` tinyint(1) UNSIGNED DEFAULT 0,
            `home_featured` tinyint(1) UNSIGNED DEFAULT 0,
            `date_add` datetime NOT NULL,
            PRIMARY KEY (`id_st_product_comment`),
            KEY `id_product` (`id_product`),
            KEY `id_customer` (`id_customer`),
            KEY `id_guest` (`id_guest`)
			) ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=utf8 ;');
        
        $return &= Db::getInstance()->execute('
			CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'st_product_comment_criterion` (
            `id_st_product_comment_criterion` int(11) unsigned NOT NULL auto_increment,
            `id_st_product_comment_criterion_type` tinyint(1) NOT NULL,
            `active` tinyint(1) NOT NULL,
            PRIMARY KEY (`id_st_product_comment_criterion`)
			) ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=utf8 ;');
            
        $return &= Db::getInstance()->execute('
			CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'st_product_comment_criterion_lang` (
            `id_st_product_comment_criterion` int(11) UNSIGNED NOT NULL,
            `id_lang` int(11) UNSIGNED NOT NULL ,
            `name` VARCHAR(64) NOT NULL ,
            PRIMARY KEY ( `id_st_product_comment_criterion` , `id_lang` )
			) ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=utf8 ;');
        
        $return &= Db::getInstance()->execute('
			CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'st_product_comment_grade` (
            `id_st_product_comment` int(10) unsigned NOT NULL,
            `id_st_product_comment_criterion` int(10) unsigned NOT NULL,
            `grade` int(10) unsigned NOT NULL,
            PRIMARY KEY (`id_st_product_comment`, `id_st_product_comment_criterion`),
            KEY `id_st_product_comment_criterion` (`id_st_product_comment_criterion`)
			) ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=utf8 ;');
            
        $return &= Db::getInstance()->execute('
			CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'st_product_comment_usefulness` (
            `id_st_product_comment` int(10) unsigned NOT NULL,
            `id_customer` int(10) unsigned NOT NULL,
            `usefulness` tinyint(1) unsigned NOT NULL,
            PRIMARY KEY (`id_st_product_comment`, `id_customer`)
			) ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=utf8 ;');
            
        $return &= Db::getInstance()->execute('
			CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'st_product_comment_report` (
            `id_st_product_comment` int(10) unsigned NOT NULL,
            `id_customer` int(10) unsigned NOT NULL,
            PRIMARY KEY (`id_st_product_comment`, `id_customer`)
			) ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=utf8 ;');
            
        $return &= Db::getInstance()->execute('
			CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'st_product_comment_tag` (
            `id_st_product_comment_tag` int(10) unsigned NOT NULL auto_increment,
            `name` VARCHAR(64) NOT NULL ,
            `id_product` int(10) unsigned NOT NULL DEFAULT 0,
            PRIMARY KEY (`id_st_product_comment_tag`)
			) ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=utf8 ;');
            
        $return &= Db::getInstance()->execute('
			CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'st_product_comment_tag_map` (
            `id_st_product_comment` int(10) unsigned NOT NULL,
            `id_st_product_comment_tag` int(10) unsigned NOT NULL,
            PRIMARY KEY (`id_st_product_comment`, `id_st_product_comment_tag`)
			) ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=utf8 ;');
            
        $return &= Db::getInstance()->execute('
			CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'st_product_comment_image` (
            `id_st_product_comment` int(10) unsigned NOT NULL,
            `image` varchar(64) NOT NULL
			) ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=utf8 ;');

        $return &= Db::getInstance()->execute('
            CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'st_product_comment_reminder` (
            `id_st_product_comment_reminder` int(11) unsigned NOT NULL auto_increment,
            `id_shop` int(10) UNSIGNED NOT NULL DEFAULT 0,
            `id_order` int(10) UNSIGNED NOT NULL DEFAULT 0,
            `reminder_count` int(10) UNSIGNED NOT NULL DEFAULT 0,
            `last_update` datetime NOT NULL,
            PRIMARY KEY (`id_st_product_comment_reminder`),
            UNIQUE KEY `id_shop_order` (`id_shop`,`id_order`)
            ) ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=utf8 ;');
        
        return $return;
    }
    
    private function initHookArray()
    {
        $this->_hooks = array(
            'Product' => array(
                array(
                    'id' => 'displayFooterProduct',
                    'val' => '1',
                    'name' => $this->l('displayFooterProduct'),
                ),
                array(
                    'id' => 'displayProductExtraContent',
                    'val' => '1',
                    'name' => $this->l('displayProductExtraContent(Product tabs)'),
                ),
                array(
                    'id' => 'displayAfterProductThumbs',
                    'val' => '1',
                    'name' => $this->l('displayAfterProductThumbs'),
                    'in_product_page' => 1,
                ),
                array(
                    'id' => 'displayProductAdditionalInfo',
                    'val' => '1',
                    'name' => $this->l('displayProductAdditionalInfo'),
                    'in_product_page' => 1,
                ),
            ),
            'Column' => array(
                array(
                    'id' => 'displayLeftColumn',
                    'val' => '1',
                    'name' => $this->l('Left column except the produt page'),
                    'in_column'=>1,
                ),
                array(
                    'id' => 'displayRightColumn',
                    'val' => '1',
                    'name' => $this->l('Right column except the produt page'),
                    'in_column'=>1,
                ),
                array(
                    'id' => 'displayLeftColumnProduct',
                    'val' => '1',
                    'name' => $this->l('Left column on the product page only'),
                    'in_column'=>1,
                ),
                array(
                    'id' => 'displayRightColumnProduct',
                    'val' => '1',
                    'name' => $this->l('Right column on the product page only'),
                    'in_column'=>1,
                ),
            ),
        );
        if ($this->_st_themes) {
            $this->_hooks['Homepage'] = array(
                array(
                    'id' => 'displayHome',
                    'val' => '1',
                    'name' => $this->l('displayHome'),
                ),
                array(
                    'id' => 'displayFullWidthTop',
                    'val' => '1',
                    'name' => $this->l('displayFullWidthTop'),
                ),
                array(
                    'id' => 'displayFullWidthTop2',
                    'val' => '1',
                    'name' => $this->l('displayFullWidthTop2'),
                ),
                array(
                    'id' => 'displayHomeTop',
                    'val' => '1',
                    'name' => $this->l('displayHomeTop'),
                ),
                array(
                    'id' => 'displayHomeBottom',
                    'val' => '1',
                    'name' => $this->l('displayHomeBottom'),
                ),
                array(
                    'id' => 'displayHomeLeft',
                    'val' => '1',
                    'name' => $this->l('displayHomeLeft'),
                ),
                array(
                    'id' => 'displayHomeRight',
                    'val' => '1',
                    'name' => $this->l('displayHomeRight'),
                ),
                array(
                    'id' => 'displayHomeFirstQuarter',
                    'val' => '1',
                    'name' => $this->l('displayHomeFirstQuarter'),
                ),
                array(
                    'id' => 'displayHomeSecondQuarter',
                    'val' => '1',
                    'name' => $this->l('displayHomeSecondQuarter'),
                ),
                array(
                    'id' => 'displayHomeThirdQuarter',
                    'val' => '1',
                    'name' => $this->l('displayHomeThirdQuarter'),
                ),
                array(
                    'id' => 'displayHomeFourthQuarter',
                    'val' => '1',
                    'name' => $this->l('displayHomeFourthQuarter'),
                ),
                array(
                    'id' => 'displayFullWidthBottom',
                    'val' => '1',
                    'name' => $this->l('displayFullWidthBottom'),
                ),
                array(
                    'id' => 'displayFooterBefore',
                    'val' => '1',
                    'name' => $this->l('displayFooterBefore'),
                ),
            );
            $this->_hooks['Footer'] = array(
                array(
                    'id' => 'displayStackedFooter1',
                    'val' => '1',
                    'name' => $this->l('displayStackedFooter1'),
                    'in_footer'=>1,
                ),
                array(
                    'id' => 'displayStackedFooter2',
                    'val' => '1',
                    'name' => $this->l('displayStackedFooter2'),
                    'in_footer'=>1,
                ),
                array(
                    'id' => 'displayStackedFooter3',
                    'val' => '1',
                    'name' => $this->l('displayStackedFooter3'),
                    'in_footer'=>1,
                ),
                array(
                    'id' => 'displayStackedFooter4',
                    'val' => '1',
                    'name' => $this->l('displayStackedFooter4'),
                    'in_footer'=>1,
                ),
                array(
                    'id' => 'displayStackedFooter5',
                    'val' => '1',
                    'name' => $this->l('displayStackedFooter5'),
                    'in_footer'=>1,
                ),
                array(
                    'id' => 'displayStackedFooter6',
                    'val' => '1',
                    'name' => $this->l('displayStackedFooter6'),
                    'in_footer'=>1,
                ),
                array(
                    'id' => 'displayFooter',
                    'val' => '1',
                    'name' => $this->l('displayFooter'),
                    'in_footer'=>1,
                ),
                array(
                    'id' => 'displayFooterAfter',
                    'val' => '1',
                    'name' => $this->l('displayFooterAfter'),
                    'in_footer'=>1,
                ),
            );
        } else {
            $this->_hooks['Homepage'] = array(
                array(
                    'id' => 'displayWrapperTop',
                    'val' => '1',
                    'name' => $this->l('displayWrapperTop'),
                ),
                array(
                    'id' => 'displayContentWrapperTop',
                    'val' => '1',
                    'name' => $this->l('displayContentWrapperTop'),
                ),
                array(
                    'id' => 'displayHome',
                    'val' => '1',
                    'name' => $this->l('displayHome'),
                ),
                array(
                    'id' => 'displayContentWrapperBottom',
                    'val' => '1',
                    'name' => $this->l('displayContentWrapperBottom'),
                ),
                array(
                    'id' => 'displayWrapperBottom',
                    'val' => '1',
                    'name' => $this->l('displayWrapperBottom'),
                ),
            );
            $this->_hooks['Footer'] = array(
                array(
                    'id' => 'displayFooterBefore',
                    'val' => '1',
                    'name' => $this->l('displayFooterBefore'),
                    'in_footer'=>1,
                ),
                array(
                    'id' => 'displayFooter',
                    'val' => '1',
                    'name' => $this->l('displayFooter'),
                    'in_footer'=>1,
                ),
                array(
                    'id' => 'displayFooterAfter',
                    'val' => '1',
                    'name' => $this->l('displayFooterAfter'),
                    'in_footer'=>1,
                ),
            );
        }
    }
    
    private function saveHook()
    {
        foreach ($this->_hooks as $key => $values) {
            if (!$key) {
                continue;
            }
            foreach ($values as $value) {
                $id_hook = Hook::getIdByName($value['id']);
                
                if (Tools::getValue($key.'_'.$value['id'])) {
                    if ($id_hook && Hook::getModulesFromHook($id_hook, $this->id)) {
                        continue;
                    }
                    if (!$this->isHookableOn($value['id'])) {
                        $this->validation_errors[] = $this->l('This module cannot be transplanted to ').$value['id'];
                    } else {
                        $this->registerHook($value['id'], Shop::getContextListShopID());
                    }
                } else {
                    if ($id_hook && Hook::getModulesFromHook($id_hook, $this->id)) {
                        $this->unregisterHook($id_hook, Shop::getContextListShopID());
                        $this->unregisterExceptions($id_hook, Shop::getContextListShopID());
                    }
                }
            }
        }
        // clear module cache to apply new data.
        Cache::clean('hook_module_list');
    }

    public function initOrderStates()
    {
        $this->order_states = array();
        foreach(OrderStateCore::getOrderStates(Context::getContext()->language->id) as $val) {
            $this->order_states[] = array(
                'id' => $val['id_order_state'],
                'name' => $val['name'],
                'val' => '1',
            );
        }
    }

    private function saveOrderStates()
    {
        foreach($this->order_states as $value) {
            if (Tools::getValue('order_state_'.$value['id'])) {
                Configuration::updateValue($this->_prefix_st.strtoupper('order_state_'.$value['id']), 1);
            } else {
                Configuration::updateValue($this->_prefix_st.strtoupper('order_state_'.$value['id']), 0);
            }
            if (Tools::getValue('gcr_order_state_'.$value['id'])) {
                Configuration::updateValue($this->_prefix_st.strtoupper('gcr_order_state_'.$value['id']), 1);
            } else {
                Configuration::updateValue($this->_prefix_st.strtoupper('gcr_order_state_'.$value['id']), 0);
            }
        }
    }

    public function uninstall()
    {
        if (!parent::uninstall()
            || !$this->uninstallDB()
        ) {
            return false;
        }
        return true;
    }
    public function sampleData()
    {
        $return = true;
        $samples = array(
            array(
                'name' => 'Quality',
            ),
            array(
                'name' => 'Fit',
            ),
        );
        foreach ($samples as $sample) {
            $module = new StProductCommentCriterionClass();
            foreach (Language::getLanguages(false) as $lang) {
                $module->name[$lang['id_lang']] = $sample['name'];
            }
            $module->id_product_comment_criterion_type = 1;
            $module->active = 1;
            $return &= $module->add();
        }
        // order status.
        foreach(OrderStateCore::getOrderStates(Context::getContext()->language->id) as $val) {
            if ($val['shipped']) {
                $return &= Configuration::updateValue($this->_prefix_st.strtoupper('order_state_'.$val['id_order_state']), 1);
            } else {
                $return &= Configuration::updateValue($this->_prefix_st.strtoupper('order_state_'.$val['id_order_state']), 0);
            }
            if ($val['template'] == 'payment') {
                $return &= Configuration::updateValue($this->_prefix_st.strtoupper('gcr_order_state_'.$val['id_order_state']), 1);
            } else {
                $return &= Configuration::updateValue($this->_prefix_st.strtoupper('gcr_order_state_'.$val['id_order_state']), 0);
            }
        }
        // pages.
        Configuration::updateValue($this->_prefix_st.strtoupper('gcr_badge_on_index'), 1);
        Configuration::updateValue($this->_prefix_st.strtoupper('gcr_badge_on_category'), 1);
        Configuration::updateValue($this->_prefix_st.strtoupper('gcr_badge_on_product'), 1);

        $languages = Language::getLanguages(false);
        $fields = array();
        foreach ($languages as $language) {
            $fields['extra_message'][$language['id_lang']] = '';
            $fields['email_review_subject'][$language['id_lang']] = 'Your review was published';
            $fields['email_reply_subject'][$language['id_lang']] = 'A new reply was published by your review.';
            $fields['email_reminder_subject'][$language['id_lang']] = 'I hope you are satisfied with your order on our shop!';
        }
        // carriers
        $carriers = Carrier::getCarriers(Context::getContext()->language->id);
        foreach($carriers as $carrier) {
            Configuration::updateValue($this->_prefix_st.'CARRIER_EST_'.$carrier['id_carrier'], 10);
        }
        foreach ($fields as $k => $v) {
            $return &= Configuration::updateValue($this->_prefix_st.strtoupper($k), $v);
        }
        $return &= Configuration::updateValue($this->_prefix_st.Tools::strtoupper('id_category_excluded'), '');
        return $return;
    }
    private function uninstallDb()
    {
        return Db::getInstance()->execute('
			DROP TABLE IF EXISTS
			`'._DB_PREFIX_.'st_product_comment`,
			`'._DB_PREFIX_.'st_product_comment_criterion`,
			`'._DB_PREFIX_.'st_product_comment_criterion_lang`,
			`'._DB_PREFIX_.'st_product_comment_grade`,
			`'._DB_PREFIX_.'st_product_comment_usefulness`,
            `'._DB_PREFIX_.'st_product_comment_report`,
            `'._DB_PREFIX_.'st_product_comment_tag`,
            `'._DB_PREFIX_.'st_product_comment_tag_map`,
			`'._DB_PREFIX_.'st_product_comment_image`');
    }
    private function _checkImageDir()
    {
        if (!file_exists(_PS_UPLOAD_DIR_.$this->name)) {
            $success = @mkdir(_PS_UPLOAD_DIR_.$this->name, self::$access_rights, true)
                        || @chmod(_PS_UPLOAD_DIR_.$this->name, self::$access_rights);
            if (!$success) {
                $this->_html .= $this->displayError('"'._PS_UPLOAD_DIR_.$this->name.'" '.$this->l('An error occurred during new folder creation'));
            }
        }
    }
    public function generateThumbnails()
    {
        foreach ($this->fields_form as &$form) {
            foreach ($form['form']['input'] as &$field) {
                if ($field['type'] == 'file') {
                    if ($bg_img = Configuration::get($this->_prefix_st.Tools::strtoupper($field['name']))) {
                        $this->fetchMediaServer($bg_img);
                        $field['image'] = '<img class="st_thumb_nail" src="'.($bg_img).'" /><p><a class="btn btn-default st_delete_image" data-field="'.$field['name'].'" href="javascript:;"><i class="icon-trash"></i>'.$this->l('Delete').'</a></p>';
                    }
                }
            }
        }
    }
    public function fetchMediaServer(&$slider)
    {
        $slider = _THEME_PROD_PIC_DIR_.$slider;
        $slider = context::getContext()->link->protocol_content.Tools::getMediaServer($slider).$slider;
    }
    protected function ajaxDeleteImage($field = '')
    {
        $result = array(
            'r' => false,
            'm' => '',
            'd' => ''
        );
        if ($field && Configuration::updateValue($this->_prefix_st.Tools::strtoupper($field), '')) {
            $result['r'] = true;
        }
        die(json_encode($result));
    }
    private function putCSVFile($data = array(), $name)
    {
        if (!$data || !$name) {
            return false;
        }
        $csv = _PS_MODULE_DIR_.$this->name.'/'.$name.'.csv';
        @unlink($csv);
        if ($fp = fopen($csv, 'w')) {
            $i = 0;
            foreach ($data as &$cur)
            {
                if ($i == 0) {
                   $cvsHeadings = array_keys($cur);  
                   // writes file header for correct encoding.
                   //fprintf($fp, chr(0xEF).chr(0xBB).chr(0xBF));

                   fputcsv($fp, $cvsHeadings, ';');
                   fputcsv($fp, $cur, ';');
                } else {      
                    fputcsv($fp, $cur, ';');
                }
                $i++;
            }
            fclose($fp);
            if (file_exists($csv)) {
                return true;
            }
        }
        return false;
    }
    public function getContent()
    {
        if (Tools::getValue('act')=='delete_image' && $field=Tools::getValue('field')) {
            return $this->ajaxDeleteImage($field);
        }
        $this->context->controller->addCSS($this->_path.'views/css/admin.css');
        $this->context->controller->addJS($this->_path.'views/js/admin.js');
        $id_st_product_comment = Tools::getValue('id_st_product_comment');
        $this->initHookArray();
        $this->_html .= $this->getConfigError() . $this->prepareNotification();
        if (Tools::isSubmit('add'.$this->name) || (Tools::isSubmit('update'.$this->name) && $id_st_product_comment)) {
            $helper = $this->renderCommentForm();
            return $this->_html.$helper->generateForm($this->fields_form);
        } elseif (Tools::isSubmit('add'.$this->name.'criterion') || Tools::isSubmit('update'.$this->name.'criterion')) {
            return $this->_html.$this->renderCriterionForm();
        } elseif (Tools::isSubmit('set'.$this->name)) {
            $content = '';
            $tabs = array();
            $tabs = array(
                array('id'  => '0,1,5,11,6', 'name' => $this->l('General settings')),
                array('id'  => '3', 'name' => $this->l('Review criteria')),
                array('id'  => '8,7', 'name' => $this->l('Home slider')),
                array('id'  => '9', 'name' => $this->l('Column block')),
                array('id'  => '10', 'name' => $this->l('Footer block')),
                array('id'  => '15,16,17', 'name' => $this->l('Email and reminder')),
                array('id'  => '18', 'name' => $this->l('Reminder stats')),
                array('id'  => '20', 'name' => $this->l('Voucher')),
                array('id'  => '12,13', 'name' => $this->l('Import/Export reviews')),
                array('id'  => '25,26', 'name' => $this->l('Google Customer Reviews')),
                array('id'  => '2', 'name' => $this->l('Advanced settings')),
            );
            $content .= $this->renderConfigForm();
            $this->smarty->assign(array(
                'identi' => 3,
                'panel_content' => $this->renderCriterionList(),
            ));
            Media::addJsDef(array(
                'id_tab_index' => Tools::getValue('id_tab_index', 0),
            ));
            $content .= $this->display(__FILE__, 'bo_panel_layout.tpl');
            $this->smarty->assign(array(
                'current_index' => AdminController::$currentIndex.'&configure='.$this->name.'&token='.Tools::getAdminTokenLite('AdminModules'),
                'bo_tabs' => $tabs,
                'bo_tab_content' => $content,
            ));
            return $this->_html.$this->display(__FILE__, 'bo_tab_layout.tpl');
        } else {
            $this->_checkImageDir();
            $this->_postProcess();
            $setting_botton = '';
            if (Tools::isSubmit('view'.$this->name)) {
                $this->_html .= $this->getNavigate($id_st_product_comment);
            } else {
                $this->smarty->assign(array(
                    'label' => $this->l('Global settings'),
                    'link' => AdminController::$currentIndex.'&configure='.$this->name.'&set'.$this->name.'&token='.Tools::getAdminTokenLite('AdminModules'),
                ));
                $setting_botton = $this->display(__FILE__, 'bo_button.tpl');
                $this->_html .= $setting_botton.$this->renderModerateLists();
            }
            return $this->_html.$setting_botton.$this->renderCommentsList();
        }
    }
    private function _postProcess()
    {
        $id_st_product_comment = Tools::getValue('id_st_product_comment');
        if (Tools::getIsset('save'.$this->name)) {
            if ($id_st_product_comment) {
                $comment = new StProductCommentClass((int)$id_st_product_comment);
            } else {
                $comment = new StProductCommentClass();
                $parent = new StProductCommentClass(Tools::getValue('id_parent'));
                $comment->id_parent = $parent->id_parent;
                $comment->id_order_detail = $parent->id_order_detail;
                $comment->id_product = $parent->id_product;
            }
            $error = array();
            $comment->copyFromPost();
            $validate = $comment->validateFields(false, true);
            if (!count($error) && $validate === true) {
                if (!$comment->id_shop) {
                    $comment->id_shop = (int)$this->context->shop->id;
                }
                if ($comment->save()) {
                    if ($grade = (int)Tools::getValue('grade'))
                    {
                        StProductCommentCriterionClass::deleteByComment($comment->id);
                        foreach(StProductCommentCriterionClass::getCriterions($this->context->language->id) as $value) {
                            $product_comment_criterion = new StProductCommentCriterionClass($value['id_st_product_comment_criterion']);
                            if ($product_comment_criterion->id)
                                $product_comment_criterion->addGrade($comment->id, $grade);
                        }
                        $comment->grade = $grade;
                        // Update Grade average of comment
                        $comment->save();
                    }
                    //
                    $old_images = StProductCommentClass::getCommentImages($comment->id, true);
                    if (count($old_images)) {
                        $image = Tools::getValue('image');
                        if (!$image || count(explode(',',$image))!=count($old_images)) {
                            StProductCommentClass::deleteImages($comment->id);
                        }
                        if($image && count(explode(',',$image))!=count($old_images)) {
                            $image_arr = explode(',',$image);
                            $pc_image_arr = array();
                            foreach ($image_arr as $img) {
                                $pc_image_arr[] = array('id_st_product_comment'=>$comment->id, 'image' => pSQL($img));
                            }
                            Db::getInstance()->insert(
                                'st_product_comment_image',
                                $pc_image_arr
                            );
                        }
                    }
                    //
                    $this->_clearCache('*');
                    Tools::redirectAdmin(AdminController::$currentIndex.'&configure='.$this->name.($comment->id_parent ? '&view'.$this->name.'&id_st_product_comment='.$comment->id_parent : '').'&conf='.($id_st_product_comment?4:3).'&token='.Tools::getAdminTokenLite('AdminModules'));
                } else {
                    Tools::redirectAdmin(AdminController::$currentIndex.'&configure='.$this->name.($id_st_product_comment ? '&update'.$this->name.'&id_st_product_comment='.$id_st_product_comment : '&add'.$this->name.'&id_parent='.$comment->id_parent).'&st_conf='.($id_st_product_comment?4:3).'&token='.Tools::getAdminTokenLite('AdminModules'));
                }
            } else {
                $url = AdminController::$currentIndex.'&configure='.$this->name.($id_st_product_comment ? '&update'.$this->name.'&id_st_product_comment='.$id_st_product_comment : '&add'.$this->name.'&id_parent='.$comment->id_parent).'&token='.Tools::getAdminTokenLite('AdminModules');
                $this->redirectWithNotification($this->displayError(str_replace('Property StProductCommentClass->', '', $validate)), $url);
            }
        } elseif (Tools::getIsset('save'.$this->name.'criterion')) {
            $id_st_product_comment_criterion = Tools::getValue('id_st_product_comment_criterion');
            $criterion = new StProductCommentCriterionClass((int)$id_st_product_comment_criterion);
            $criterion->copyFromPost();
            if ($criterion->validateFields(false) && $criterion->save()) {
                $this->_clearCache('*');
                Tools::redirectAdmin(AdminController::$currentIndex.'&configure='.$this->name.'&set'.$this->name.'&conf='.($id_st_product_comment_criterion?4:3).'&token='.Tools::getAdminTokenLite('AdminModules').'#1');
            } else {
                Tools::redirectAdmin(AdminController::$currentIndex.'&configure='.$this->name.($id_st_product_comment_criterion ? '&update'.$this->name.'criterion&id_st_product_comment_criterion='.$id_st_product_comment_criterion : 'add'.$this->name.'criterion').'&st_conf='.($id_st_product_comment_criterion?4:3).'&token='.Tools::getAdminTokenLite('AdminModules').'#1');
            }
        } elseif (Tools::getIsset('save'.$this->name.'setting')) {
            $this->initConfigFormFields();
            foreach ($this->fields_form as $form) {
                foreach ($form['form']['input'] as $field) {
                    if (isset($field['validation'])) {
                        $errors = array();
                        $value = Tools::getValue($field['name']);
                        if (isset($field['required']) && $field['required'] && $value==false && (string)$value != '0') {
                            $errors[] = sprintf($this->l('Field "%s" is required.'), $field['label']);
                        } elseif ($value) {
                            $field_validation = $field['validation'];
                            if ($field_validation == 'isColor' && !self::isColor($value)) {
                                $errors[] = sprintf($this->l('Field "%s" is invalid.'), $field['label']);
                            } elseif (!Validate::$field_validation($value)) {
                                $errors[] = sprintf($this->l('Field "%s" is invalid.'), $field['label']);
                            }
                        }
                        // Set default value
                        if ($value === false && isset($field['default_value'])) {
                            $value = $field['default_value'];
                        }
                        
                        if ($field['name']=='limit' && $value>20) {
                            $value=20;
                        }
                        
                        if (count($errors)) {
                            $this->validation_errors = array_merge($this->validation_errors, $errors);
                        } elseif ($value==false) {
                            switch ($field['validation']) {
                                case 'isUnsignedId':
                                case 'isUnsignedInt':
                                case 'isInt':
                                case 'isBool':
                                    $value = 0;
                                    break;
                                default:
                                    $value = '';
                                    break;
                            }
                            Configuration::updateValue($this->_prefix_st.Tools::strtoupper($field['name']), $value);
                        } else {
                            Configuration::updateValue($this->_prefix_st.Tools::strtoupper($field['name']), $value);
                        }
                    }
                }
            }
            $this->updateCatePerRow();
            if (Tools::getValue('Product_displayFooterProduct') && Tools::getValue('Product_displayProductExtraContent')) {
                unset($_POST['Product_displayFooterProduct']);
            }
            $this->saveHook();
            $this->saveOrderStates();
            // Exclude categories
            if ($id_category = Tools::getValue('id_category')) {
                Configuration::updateValue($this->_prefix_st.Tools::strtoupper('id_category_excluded'), implode(',', $id_category));
            } else {
                Configuration::updateValue($this->_prefix_st.Tools::strtoupper('id_category_excluded'), '');
            }
            // Update carrier estimated time.
            foreach($_POST as $k => $v) {
                if (strpos($k, 'carrier_est_') !== false) {
                    Configuration::updateValue($this->_prefix_st.Tools::strtoupper($k), (int)$v);
                }
            }
            // Update pages.
            foreach($this->_pages as $v) {
                if (Tools::getValue('gcr_badge_on_'.$v['id'])) {
                    Configuration::updateValue($this->_prefix_st.strtoupper('gcr_badge_on_'.$v['id']), 1);
                } else {
                    Configuration::updateValue($this->_prefix_st.strtoupper('gcr_badge_on_'.$v['id']), 0);
                }
            }
            foreach ($this->fields_form as $form) {
                foreach ($form['form']['input'] as $field) {
                    if ($field['type'] == 'file') {
                        $res = $this->stUploadImage($field['name']);
                        if (count($res['error'])) {
                            $this->validation_errors = array_merge($this->validation_errors, $res['error']);
                        } elseif ($res['image']) {
                            Configuration::updateValue($this->_prefix_st.Tools::strtoupper($field['name']), $res['image']);
                        }
                    }
                }
            }
            $languages = Language::getLanguages(false);
            $fields = array();
            foreach ($languages as $language) {
                foreach($this->lang_field as $v) {
                    $fields[$v][$language['id_lang']] = Tools::getValue($v.'_'.$language['id_lang']);
                }
            }
            foreach ($fields as $k => $v) {
                Configuration::updateValue($this->_prefix_st.strtoupper($k), $v);
            }
            $this->importComments();

            $this->_clearCache('*');
            if (count($this->validation_errors)) {
                $errors = $this->displayError(implode('<br/>', $this->validation_errors));
                $url = AdminController::$currentIndex.'&configure='.$this->name.'&set'.$this->name.'&id_tab_index='.(int)Tools::getValue('id_tab_index', 0).'&token='.Tools::getAdminTokenLite('AdminModules');
                $this->redirectWithNotification($errors, $url);
            } else {
                Tools::redirectAdmin(AdminController::$currentIndex.'&configure='.$this->name.'&set'.$this->name.'&conf=6&id_tab_index='.(int)Tools::getValue('id_tab_index', 0).'&token='.Tools::getAdminTokenLite('AdminModules'));
            }
        } elseif(Tools::getValue('export')) {
            $this->exportComments();
        } elseif ((Tools::isSubmit('status'.$this->name))) {
            $comment = new StProductCommentClass((int)$id_st_product_comment);
            if ($comment->id) {
                if ($comment->id_parent) {
                    $comment->setFieldsToUpdate(array('validate' => true));
                    $comment->validate = !(int)$comment->validate;
                } else {
                    $comment->setFieldsToUpdate(array('home_featured' => true));
                    $comment->home_featured = !(int)$comment->home_featured;
                }
                
                $comment->update(false);
                $this->_clearCache('*');
                Tools::redirectAdmin(AdminController::$currentIndex.'&configure='.$this->name.($comment->id_parent ? '&view'.$this->name.'&id_st_product_comment='.$comment->id_parent : '').'&conf=5&token='.Tools::getAdminTokenLite('AdminModules'));
            } else {
                Tools::redirectAdmin(AdminController::$currentIndex.'&configure='.$this->name.($comment->id_parent ? '&view'.$this->name.'&id_st_product_comment='.$comment->id_parent : '').'&st_conf=5&token='.Tools::getAdminTokenLite('AdminModules'));
            }
        } elseif ((Tools::isSubmit('status'.$this->name.'criterion'))) {
            $id_st_product_comment_criterion = Tools::getValue('id_st_product_comment_criterion');
            $criterion = new StProductCommentCriterionClass((int)$id_st_product_comment_criterion);
            if ($criterion->id && $criterion->toggleStatus()) {
                $this->_clearCache('*');
                Tools::redirectAdmin(AdminController::$currentIndex.'&configure='.$this->name.'&set'.$this->name.'&conf=5&token='.Tools::getAdminTokenLite('AdminModules').'#1');
            } else {
                Tools::redirectAdmin(AdminController::$currentIndex.'&configure='.$this->name.'&set'.$this->name.'&st_conf=5&token='.Tools::getAdminTokenLite('AdminModules').'#1');
            }
        } elseif ((Tools::isSubmit('delete'.$this->name))) {
            $comment = new StProductCommentClass((int)$id_st_product_comment);
            if ($comment->id && $comment->delete()) {
                $this->_clearCache('*');
                Tools::redirectAdmin(AdminController::$currentIndex.'&configure='.$this->name.($comment->id_parent ? '&view'.$this->name.'&conf=1&id_st_product_comment='.$comment->id_parent : '').'&token='.Tools::getAdminTokenLite('AdminModules'));
            } else {
                Tools::redirectAdmin(AdminController::$currentIndex.'&configure='.$this->name.($comment->id_parent ? '&view'.$this->name.'&id_st_product_comment='.$comment->id_parent : '').'&st_conf=1&token='.Tools::getAdminTokenLite('AdminModules'));
            }
        } elseif ((Tools::isSubmit('delete'.$this->name.'criterion'))) {
            $id_st_product_comment_criterion = Tools::getValue('id_st_product_comment_criterion');
            $criterion = new StProductCommentCriterionClass((int)$id_st_product_comment_criterion);
            if ($criterion->id && $criterion->delete()) {
                $this->_clearCache('*');
                Tools::redirectAdmin(AdminController::$currentIndex.'&configure='.$this->name.'&set'.$this->name.'&conf=1&token='.Tools::getAdminTokenLite('AdminModules').'#1');
            } else {
                Tools::redirectAdmin(AdminController::$currentIndex.'&configure='.$this->name.'&set'.$this->name.'&st_conf=1&token='.Tools::getAdminTokenLite('AdminModules').'#1');
            }
        } elseif ($id_product_comment = (int) Tools::getValue('approveComment')) {
            $comment = new StProductCommentClass($id_product_comment);
            $comment->validate();
            $this->_clearCache('*');
            Tools::redirectAdmin(AdminController::$currentIndex.'&configure='.$this->name.($comment->id_parent ? '&view'.$this->name.'&id_st_product_comment='.$comment->id_parent : '').'&conf=5&token='.Tools::getAdminTokenLite('AdminModules'));
        } elseif ($id_product_comment = (int) Tools::getValue('noabuseComment')) {
            StProductCommentClass::deleteReports($id_product_comment);
            $this->_clearCache('*');
            Tools::redirectAdmin(AdminController::$currentIndex.'&configure='.$this->name.($comment->id_parent ? '&view'.$this->name.'&id_st_product_comment='.$comment->id_parent : '').'&token='.Tools::getAdminTokenLite('AdminModules'));
        }
    }
    public function updateCatePerRow() {
        foreach ($this->fields_default_stsn as $k => $v) {
            Configuration::updateValue($this->_prefix_stsn.Tools::strtoupper($k), (int)Tools::getValue($k));
        }
    }
    public function importComments()
    {
        // Import data from CSV.
        if (isset($_FILES['csv_file']) && $_FILES['csv_file'] && !empty($_FILES['csv_file']['tmp_name']) && $_FILES['csv_file']['error'] == 0) {
            $ext = strtolower(substr(strrchr($_FILES['csv_file']['name'], '.'), 1));
            if ($ext != 'csv') {
                $this->validation_errors[] = $this->l('Please upload csv file.');
            }
            $file = _PS_MODULE_DIR_.$this->name.'/import/'.date('YmdH').'.csv';
            if (!move_uploaded_file($_FILES['csv_file']['tmp_name'], $file)) {
                $this->validation_errors[] = $this->l('Move upload file failed.');
            }
            if (file_exists($file)) {
                $separator = (int)Configuration::get($this->_prefix_st.'SEPARATOR');
                if ($separator == 1) {
                    $separator = ',';
                } elseif ($separator == 2) {
                    $separator = "\t";
                } else {
                    $separator = ';';
                }
                if ($fp = fopen($file, 'r')) {
                    set_time_limit(6*60);
                    $n = 0;
                    while (!feof($fp)) {
                        if (!$row = fgetcsv($fp, 8192, $separator)) {
                            continue;
                        }
                        $n++;
                        if ($n == 1) {
                            continue;
                        }
                        // [0]Comment ID;[1]Customer;[2]Rating;[3]Product ID;[4]Status;[5]comment;[6]Add time
                        $comment_id = (int)$row[0];
                        if ($comment_id) {
                            $comment = new StProductCommentClass($comment_id);
                        } else {
                            $comment = new StProductCommentClass();
                        }
                        $comment->id_parent = 0;
                        $comment->id_shop = (int)$this->context->shop->id;
                        $comment->id_order_detail = 0;
                        if ((int)$row[1] > 0) {
                            $comment->id_customer = (int)$row[1];
                            $customer = new Customer($comment->id_customer);
                            if ($customer->id) {
                                $comment->customer_name = $customer->firstname.' '.$customer->lastname;
                            }
                        } else {
                            $comment->customer_name = pSQL($row[1]);
                        }
                        $comment->id_product = (int)$row[3];
                        $comment->validate = (int)$row[4];
                        $comment->content = pSQL($row[5]);
                        $comment->date_add = pSQL($row[6]);
                        $comment->is_admin = 1;

                        if ($comment->save(false, false)) {
                            if ($grade = (int)$row[2])
                            {
                                if ($grade > 5 || $grade <= 0) {
                                    $grade = 5;
                                }
                                StProductCommentCriterionClass::deleteByComment($comment->id);
                                foreach(StProductCommentCriterionClass::getCriterions($this->context->language->id) as $value) {
                                    $product_comment_criterion = new StProductCommentCriterionClass($value['id_st_product_comment_criterion']);
                                    if ($product_comment_criterion->id) {
                                        $product_comment_criterion->addGrade($comment->id, $grade);
                                    }
                                }
                                $comment->grade = $grade;
                                // Update Grade average of comment
                                $comment->save();
                            }
                        }
                    }
                    fclose($fp);
                    $url = AdminController::$currentIndex.'&configure='.$this->name.'&set'.$this->name.'&token='.Tools::getAdminTokenLite('AdminModules');
                    $this->redirectWithNotification($this->displayConfirmation(sprintf($this->l('Imported %s reviews.'), $n)), $url);
                }
            } else {
                $url = AdminController::$currentIndex.'&configure='.$this->name.'&set'.$this->name.'&token='.Tools::getAdminTokenLite('AdminModules');
                $error = sprintf($this->l('Upload file failed, plase ensure the folder %s is writable.'), dirname($file));
                $this->redirectWithNotification($this->displayError($error), $url);
            }
        }
    }
    public function exportComments()
    {
        $comments = StProductCommentClass::getByValidate(0, 0, true);
        $csv = '"Comment ID";Customer;Rating;"Product ID";Status;comment;"Add time"';
        foreach($comments as $comment) {
            $csv .= "\n";
            $csv .= '"'.(int)$comment['id_st_product_comment'].'";';
            $csv .= '"'.$comment['customer_name'].'";';
            $csv .= '"'.$comment['grade'].'";';
            $csv .= '"'.$comment['id_product'].'";';
            $csv .= '"'.$comment['validate'].'";';
            $csv .= '"'.str_replace('"', '\'', $comment['content']).'";';
            $csv .= '"'.$comment['date_add'].'"';
        }
        header("Content-type: text/x-csv");
        header("Content-Disposition: attachment; filename='product-comments-".date('Y-m-d').".csv'");
        echo $csv;die;
    }
    protected function renderModerateLists()
    {
        $return = '';
        if (Configuration::get($this->_prefix_st.'MODERATE')) {
            $comments = StProductCommentClass::getByValidate(0);

            $fields_list = $this->getStandardFieldList();
            unset($fields_list['unapproved']);
            $helper = new HelperList();
            $helper->shopLinkType = '';
            $helper->simple_header = true;
            $helper->actions = array('approve', 'delete');
            $helper->show_toolbar = true;
            $helper->module = $this;
            $helper->listTotal = count($comments);
            $helper->identifier = 'id_st_product_comment';
            $helper->title = $this->l('Reviews waiting for approval');
            $helper->table = $this->name;
            $helper->token = Tools::getAdminTokenLite('AdminModules');
            $helper->currentIndex = AdminController::$currentIndex.'&configure='.$this->name;

            $return .= $helper->generateList($comments, $fields_list);
        }

        $comments = StProductCommentClass::getReportedComments();

        $fields_list = $this->getStandardFieldList();
        unset($fields_list['unapproved']);
        $helper = new HelperList();
        $helper->shopLinkType = '';
        $helper->simple_header = true;
        $helper->actions = array('delete', 'noabuse');
        $helper->show_toolbar = true;
        $helper->module = $this;
        $helper->listTotal = count($comments);
        $helper->identifier = 'id_st_product_comment';
        $helper->title = $this->l('Reported Reviews');
        $helper->table = $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex.'&configure='.$this->name;

        $return .= $helper->generateList($comments, $fields_list);

        return $return;
    }
    protected function renderCriterionList()
    {
        $criterions = StProductCommentCriterionClass::getCriterions($this->context->language->id, false, false);

        $fields_list = array(
            'id_st_product_comment_criterion' => array(
                'title' => $this->l('ID'),
                'type' => 'text',
            ),
            'name' => array(
                'title' => $this->l('Name'),
                'type' => 'text',
            ),
            'active' => array(
                'title' => $this->l('Status'),
                'active' => 'status',
                'type' => 'bool',
            ),
        );

        $helper = new HelperList();
        $helper->shopLinkType = '';
        $helper->simple_header = false;
        $helper->actions = array('edit', 'delete');
        $helper->show_toolbar = true;
        $helper->toolbar_btn['new'] = array(
            'href' => $this->context->link->getAdminLink('AdminModules').'&configure='.$this->name.'&module_name='.$this->name.'&addstproductcommentscriterion',
            'desc' => $this->l('Add New Criterion'),
        );
        $helper->module = $this;
        $helper->identifier = 'id_st_product_comment_criterion';
        $helper->title = $this->l('Review Criteria');
        $helper->table = $this->name.'criterion';
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex.'&configure='.$this->name;

        return $helper->generateList($criterions, $fields_list);
    }
    protected function renderCommentsList()
    {
        $id_st_product_comment = Tools::getValue('id_st_product_comment', 0);
        $nbr_comments = StProductCommentClass::getByValidate(1, $id_st_product_comment, $id_st_product_comment ? true : !Configuration::get($this->_prefix_st.'MODERATE'), true);
        $fields_list = $this->getStandardFieldList();
        
        $helper = new HelperList();
        $helper->shopLinkType = '';
        $helper->simple_header = false;
        $helper->show_filters = false;
        $helper->actions = array('reply','edit','delete');
        $helper->show_toolbar = true;
        $helper->module = $this;
        $helper->listTotal = $nbr_comments;
        $helper->identifier = 'id_st_product_comment';
        if (Tools::isSubmit('view'.$this->name)) {
            $helper->title = $this->l('Replies');
        } else {
            $helper->title = $this->l('Approved Reviews');
        }
        $helper->table = $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex.'&configure='.$this->name;
        if ($id_st_product_comment) {
            $helper->toolbar_btn['new'] =  array(
                'href' => AdminController::$currentIndex.'&configure='.$this->name.'&add'.$this->name.'&id_parent='.$id_st_product_comment.'&token='.Tools::getAdminTokenLite('AdminModules'),
                'desc' => $this->l('Add a reply'),
            );
            $parent = new StProductCommentClass($id_st_product_comment);
            $helper->toolbar_btn['back'] =  array(
                'href' => AdminController::$currentIndex.'&configure='.$this->name.($parent->id_parent ? '&id_st_product_comment='.$parent->id_parent.'&view'.$this->name : '').'&token='.Tools::getAdminTokenLite('AdminModules'),
                'desc' => $this->l('Back to list')
            );
            unset($fields_list['grade'], $fields_list['home_featured']);
            $fields_list['validate'] = array(
                'title' => $this->l('Approved'),
                'active' => 'status',
                'type' => 'bool',
                );
        } else {
            $helper->toolbar_btn['new'] =  array(
                'href' => AdminController::$currentIndex.'&configure='.$this->name.'&add'.$this->name.'&id_parent=0&token='.Tools::getAdminTokenLite('AdminModules'),
                'desc' => $this->l('Add new comment'),
            );
            $helper->toolbar_btn['edit'] =  array(
                'href' => AdminController::$currentIndex.'&configure='.$this->name.'&set'.$this->name.'&id_parent='.$id_st_product_comment.'&token='.Tools::getAdminTokenLite('AdminModules'),
                'desc' => $this->l('Settings'),
            );
        }
        
        /* Paginate the result */
        $page = ($page = Tools::getValue('submitFilter'.$helper->table)) ? $page : 1;
        $pagination = ($pagination = Tools::getValue($helper->table.'_pagination')) ? $pagination : 30;
        $comments = StProductCommentClass::getByValidate(1, $id_st_product_comment, $id_st_product_comment ? true : !Configuration::get($this->_prefix_st.'MODERATE'), false, $page, $pagination);
        
        ///$comments = $this->pagination($comments, $page, $pagination);

        return $helper->generateList($comments, $fields_list);
    }
    public function getStandardFieldList()
    {
        return array(
            'id_st_product_comment' => array(
                'title' => $this->l('ID'),
                'type' => 'text',
                ),
            'content' => array(
                'title' => $this->l('Content'),
                'type' => 'text',
                'callback' => 'showContent',
                'callback_object' => $this,
                ),
            'name' => array(
                'title' => $this->l('Product'),
                'type' => 'text',
                ),
            'unapproved' => array(
                'title' => $this->l('Unapproved replies'),
                'type' => 'text',
                'search' => false,
                'callback' => 'showUnapproved',
                'callback_object' => $this,
                ),
            'home_featured' => array(
                'title' => $this->l('Home featured'),
                'active' => 'status',
                'type' => 'bool',
                ),
            'date_add' => array(
                'title' => $this->l('Date Add'),
                'type' => 'date',
                'search' => false,
                ),
        );
    }
    public function showUnapproved($value, $row)
    {
        return '<label class="circle" title="'.$this->l('Unapproved replies').'">'.$row['unapproved'].'</label>';
    }
    public function showContent($value, $row)
    {
        $html = $value.'<br>'.
            (!Tools::getValue('id_st_product_comment') ? $this->l('Rating: ').$row['grade'].' / 5, ' : '').
            $this->l('Customer name: ').$row['customer_name'];
        return $html;
    }
    public function displayApproveLink($token, $id, $name = null)
    {
        $this->smarty->assign(array(
            'href' => $this->context->link->getAdminLink('AdminModules').'&configure='.$this->name.'&module_name='.$this->name.'&approveComment='.$id,
            'action' => $this->l('Approve'),
        ));

        return $this->display(__FILE__, 'views/templates/admin/list_action_approve.tpl');
    }
    public function displayNoabuseLink($token, $id, $name = null)
    {
        $this->smarty->assign(array(
            'href' => $this->context->link->getAdminLink('AdminModules').'&configure='.$this->name.'&module_name='.$this->name.'&noabuseComment='.$id,
            'action' => $this->l('Not abusive'),
        ));

        return $this->display(__FILE__, 'views/templates/admin/list_action_noabuse.tpl');
    }
    public function displayReplyLink($token, $id, $name = null)
    {
        $this->smarty->assign(array(
            'href' => $this->context->link->getAdminLink('AdminModules').'&configure='.$this->name.'&view'.$this->name.'&id_st_product_comment='.$id,
            'action' => $this->l('Reply'),
        ));

        return $this->display(__FILE__, 'views/templates/admin/list_action_reply.tpl');
    }
    public function pagination($comments, $page = 1, $pagination = 50)
    {
        if (count($comments) > $pagination) {
            $comments = array_slice($comments, $pagination * ($page - 1), $pagination);
        }
        return $comments;
    }
    protected function initConfigFormFields()
    {
        $this->initOrderStates();
        $fields = $this->getFormFields();
        $this->fields_form[0]['form'] = array(
            'legend' => array(
                'title' => $this->displayName,
                'icon' => 'icon-cogs'
            ),
            'input' => $fields['general'],
            'submit' => array(
                'title' => $this->l('Save all', array(), 'Admin.Actions'),
            )
        );
        
        $this->fields_form[5]['form'] = array(
            'legend' => array(
                'title' => $this->l('Testimonial Page'),
                'icon' => 'icon-cogs'
            ),
            'input' => $fields['testimonial'],
            'submit' => array(
                'title' => $this->l('Save all', array(), 'Admin.Actions')
            ),
        );
        
        $this->fields_form[2]['form'] = array(
            'legend' => array(
                'title' => $this->l('Advanced setting'),
                'icon' => 'icon-cogs'
            ),
            'input' => $fields['advanced'],
            'submit' => array(
                'title' => $this->l('Save all', array(), 'Admin.Actions')
            ),
        );
        
        $this->fields_form[6]['form'] = array(
            'legend' => array(
                'title' => $this->l('Product Page'),
                'icon' => 'icon-cogs'
            ),
            'input' => $fields['product'],
            'submit' => array(
                'title' => $this->l('Save all', array(), 'Admin.Actions')
            ),
        );
        $this->fields_form[11]['form'] = array(
            'legend' => array(
                'title' => $this->l('Footer product'),
                'icon' => 'icon-cogs'
            ),
            'input' => $fields['footer_product'],
            'submit' => array(
                'title' => $this->l('Save all', array(), 'Admin.Actions')
            ),
        );
        /** Import reviews **/
        $this->fields_form[12]['form'] = array(
            'legend' => array(
                'title' => $this->l('Import comments from CSV file'),
                'icon' => 'icon-cogs'
            ),
            'input' => $fields['import'],
            'desc' => array(
                $this->l('You can specify the separator, if your CSV file was separated by semicolon, just choose Semicolon; if the file was separated by comma, pleaes choose Comma. incorrect separator will cause the comments can\'t be imported properly.'),
                $this->l('If you don\'t know what the separator the CSV file uses, you need to contact the data provider.'),
                $this->l('The template CSV was separated by semicolon.'),
            ),
            'submit' => array(
                'title' => $this->l('Import')
            ),
        );
        /** export reviews **/
        $this->fields_form[13]['form'] = array(
            'legend' => array(
                'title' => $this->l('Export comments to a CSV file'),
                'icon' => 'icon-cogs'
            ),
            'input' => $fields['export'],
            'desc' => array(
                $this->l('To export all product reviews, click the "Export comments" button, a CSV file will be generated and download automatically.'),
                $this->l('The number on the export button is the total comments will be expored. if the number is 0, there isn\'t any comment in the list.'),
                $this->l('CSV content was separated by semicolon in the file.'),
            )
        );
        /** Email settings **/
        $this->fields_form[15]['form'] = array(
            'legend' => array(
                'title' => $this->l('Email settings'),
                'icon' => 'icon-cogs'
            ),
            'input' => $fields['email'],
            'submit' => array(
                'title' => $this->l('Save all', array(), 'Admin.Actions')
            ),
            'desc' => array(
                $this->l('You can decide how to use the email notification on the section.'),
                $this->l('You can translate the email subject on the settings directly, if you want to change email content, please go to BO > Translation > Email template translations page. all templates are located in the "stproductcomments" section.'),
            )
        );
        /** Reminder setting **/
        $this->fields_form[16]['form'] = array(
            'legend' => array(
                'title' => $this->l('Reminder settings'),
                'icon' => 'icon-cogs'
            ),
            'input' => $fields['reminder'],
            'submit' => array(
                'title' => $this->l('Save all', array(), 'Admin.Actions')
            ),
            'desc' => array(
                $this->l('The settings on the section will configure how to send reminder to customers who purchased your products.'),
                $this->l('Delay for sending reminder email: how long begin to send reminder after customer placed orders.'),
                $this->l('Order states: specify which state order will be gotten email reminder.'),
                $this->l('Exclude categories: if there are any order products are on the selected categories, no reminder for the order. For example: order #1 has product A, B and c, if B was associated one of the selected excluded categories, no reminder for the order #1.'),
            )
        );
        /** Crob job **/
        $this->fields_form[17]['form'] = array(
            'legend' => array(
                'title' => $this->l('Cron settings'),
                'icon' => 'icon-cogs'
            ),
            'input' => $fields['cron'],
            'submit' => array(
                'title' => $this->l('Save all', array(), 'Admin.Actions')
            ),
            'desc' => array(
                $this->l('The setting will send batch emails to customers to reminder them to write a review automatically.'),
                $this->l('You need to create a CRON job in your sever and paste the cron URL to run in schedule.'),
                $this->l('You must configure the CRON job to run once a day, for example, run the jon at 12:00 am everyday.'),
                $this->l('If you are using multistore, please switch to each store to copy the cron URL, and create a cron job for each URL.'),
            )
        );
        /** Voucher **/
        $this->fields_form[18]['form'] = array(
            'legend' => array(
                'title' => $this->l('Reminder stats'),
                'icon' => 'icon-cogs'
            ),
            'input' => array(
                array(
                    'type' => 'html',
                    'label' => $this->l(''),
                    'name' => $this->displayReminderStats(),
                    'form_group_class' => 'reminder_stats',
                ),
            ),
        );
        /** Voucher **/
        $this->fields_form[20]['form'] = array(
            'legend' => array(
                'title' => $this->l('Voucher settings'),
                'icon' => 'icon-cogs'
            ),
            'input' => $fields['voucher'],
            'submit' => array(
                'title' => $this->l('Save all', array(), 'Admin.Actions')
            ),
            'desc' => array(
                $this->l('Attach a voucher code to your customer in the reminder email. Customers are more willing to rewrite reviews when geting the voucher.'),
                $this->l('If you didn\'t add voucher, you need to create it firstly, then add the voucher on the "Voucher code" field.'),
            )
        );

        $this->fields_form[25]['form'] = array(
            'legend' => array(
                'title' => $this->l('Preparation'),
                'icon' => 'icon-cogs'
            ),
            'input' => array(),
            'desc' => array(
                $this->l('Google Customer Reviews is a free program that lets you collect feedback from users who’ve made a purchase on your site. Ratings from Google Customer Reviews apply to your seller ratings eligibility. Seller ratings appear on Search Ads, in Shopping ads.'),
                sprintf($this->l('%sTo let the module can work fine on your site, please take the following stpes before configuration the module:%s'), '<strong>','</strong>'),
                sprintf($this->l('%s1. Enable the reviews feature on the Google Merchant Center, refer to HOW TO GET START here: %s https://support.google.com/merchants/answer/7188525?hl=en %s%s'), '<strong>','<a href="https://support.google.com/merchants/answer/7188525?hl=en" target="_blank" style="color:#f00;">','</a>','</strong>'),
                sprintf($this->l('%s2. Google requires your confirmation page of checkout flow has SSL enabled, all confirmation pages must use HTTPS (not HTTP). so please ensure your site installed a SSL certificate and enable SSL.%s'), '<strong>','</strong>'),
                $this->l('Why my Google Customer Reviews badge does\'t appear? In most cases, you must have at least 100 reviews in a particular country during the past year for your seller rating to appear on your badge.'),
                sprintf($this->l('When the seller rating appear on Google shop and Google AD? In most cases, seller ratings in a country show only when a business has 150 unique reviews from that same country and a composite rating of 3.5 stars or higher. Reviews from users in a given country will contribute to the business\' rating only in that country. see official documentation: %shttps://support.google.com/google-ads/answer/2375474%s'), '<a href="https://support.google.com/google-ads/answer/2375474", target="_blank">', '</a>'),
            ),
        );

        $this->fields_form[26]['form'] = array(
            'legend' => array(
                'title' => $this->l('Settings'),
                'icon' => 'icon-cogs'
            ),
            'input' => $fields['google_review'],
            'submit' => array(
                'title' => $this->l('Save all', array(), 'Admin.Actions')
            ),
            'desc' => array(
                $this->l('Google requires the Estimated delivery date to send email to the customer, so you will indicate it here how long each carrier will delivery to the customer. the "Estimated delivery date" equal "order processing days" + "carrier delivery time" there.'),
                $this->l('For example, the order processing days is 3, the Estimated delivery date is 7, if a customer placed an order on 1 Jan, then he will receive Google survey email on 11 Jan.')
            )
        );
        
        $this->fields_form[1]['form'] = array(
            'legend' => array(
                'title' => $this->l('Hook manager'),
                'icon' => 'icon-cogs'
            ),
            'desc' => $this->l('DO NOT hook the module to the "displayFooterProduct" and "displayProductExtraContent" at the same time.'),
            'input' => $fields['hook'],
            'submit' => array(
                'title' => $this->l('Save all', array(), 'Admin.Actions')
            ),
        );
        
        $this->fields_form[8]['form'] = array(
            'legend' => array(
                'title' => $this->l('Home slider'),
                'icon'  => 'icon-cogs'
            ),
            'input' => $fields['home_slider'],
            'submit' => array(
                'title' => $this->l('Save all'),
            ),
        );
        
        $this->fields_form[7]['form'] = array(
            'legend' => array(
                'title' => $this->l('Advanced settings'),
                'icon'  => 'icon-cogs'
            ),
            'input' => $fields['setting'],
            'submit' => array(
                'title' => $this->l('Save all'),
            ),
        );
        
        $this->fields_form[9]['form'] = array(
            'legend' => array(
                'title' => $this->l('Column'),
                'icon'  => 'icon-cogs'
            ),
            'input' => $fields['column'],
            'submit' => array(
                'title' => $this->l('Save all'),
            ),
        );
        
        $this->fields_form[10]['form'] = array(
            'legend' => array(
                'title' => $this->l('Footer'),
                'icon'  => 'icon-cogs'
            ),
            'input' => $fields['footer'],
            'submit' => array(
                'title' => $this->l('Save all'),
            ),
        );
        $back_to_list = array(
            'type' => 'go_back_to_list',
            'label' => '',
            'name' => AdminController::$currentIndex.'&configure='.$this->name.'&token='.Tools::getValue('token'),
        );
        $this->fields_form[0]['form']['input'][] = $back_to_list;
        $this->fields_form[1]['form']['input'][] = $back_to_list;
        $this->fields_form[2]['form']['input'][] = $back_to_list;
        $this->fields_form[5]['form']['input'][] = $back_to_list;
        $this->fields_form[6]['form']['input'][] = $back_to_list;
        $this->fields_form[7]['form']['input'][] = $back_to_list;
        $this->fields_form[8]['form']['input'][] = $back_to_list;
        $this->fields_form[9]['form']['input'][] = $back_to_list;
        $this->fields_form[10]['form']['input'][] = $back_to_list;
    }
    public function displayReminderStats()
    {
        $data = StProductCommentClass::getReminder();
        $html = '';
        $html = '<table class="table '.$this->name.' tab-stats">';
        $html .= '
            <th width="33%">'.$this->l('Order ID').'</th>
            <th width="33%">'.$this->l('Reminder count').'</th>
            <th width="34%">'.$this->l('Last reminder time').'</th>
            </tr>';
        foreach($data as $row) {
            $html .= '<tr>
                <td>'.$row['id_order'].'</td>
                <td>'.($row['reminder_count']).'</td>
                <td>'.$row['last_update'].'</td>
                </tr>';
        }
        $html .= '</table>';
        return $html;
    }
    protected function renderConfigForm()
    {
        $this->initConfigFormFields();
        $this->generateThumbnails();
        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->module = $this;
        $helper->table =  $this->table;
        $lang = new Language((int)Configuration::get('PS_LANG_DEFAULT'));
        $helper->default_form_language = $lang->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ? Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') : 0;
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'save'.$this->name.'setting';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false).'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFieldsValues(),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id
        );
        return $helper->generateForm($this->fields_form);
    }
    public function renderCommentForm()
    {
        $id_st_product_comment = Tools::getValue('id_st_product_comment');
        $id_parent = Tools::getValue('id_parent');
        $comment = new StProductCommentClass($id_st_product_comment);
        $pc_images = array('str'=>array(),'data'=>array());
        if (!$comment->id) {
            $comment->id_parent = (int)$id_parent;
            $comment->is_admin = 1;
        } else {
            $images = StProductCommentClass::getCommentImages($comment->id, true);
            if($images) {
                $pc_images['str'] = implode(',', $images);
                foreach($images as $image) {
                    $pc_images['data'][] = array(
                        'name' => $image,
                        'url' => context::getContext()->link->protocol_content.Tools::getMediaServer(_THEME_PROD_PIC_DIR_.$image)._THEME_PROD_PIC_DIR_.$image,
                        );
                }
            }
        }
        $this->fields_form[0]['form'] = array(
            'legend' => array(
                'title' => $this->l('Product reviews'),
                'icon' => 'icon-tags',
            ),
            'input' => array(
                array(
                    'type' => 'text',
                    'label' => $this->l('Customer Name:'),
                    'name' => 'customer_name',
                ),
                /*array(
                    'type' => 'text',
                    'label' => $this->l('Review title:'),
                    'name' => 'title',
                ),*/
                'id_product' => array(
                    'type' => 'text',
                    'label' => $this->l('Prodcut ID:'),
                    'name' => 'id_product',
                ),
                'grade' => array(
                    'type' => 'text',
                    'label' => $this->l('Stars:'),
                    'name' => 'grade',
                ),
                array(
                    'type' => 'textarea',
                    'id'   => 'cm_content',
                    'label' => $this->l('Content:'),
                    'name' => 'content',
                    'required' => true,
                    'rows' => 8,
                    'cols' => 60,
                 ),
                array(
                    'type' => 'pc_images',
                    'label' => $this->l('Images:'),
                    'name' => 'image',
                    'values' => $pc_images,
                ),
                 array(
                    'type' => 'switch',
                    'label' => $this->l('Approved:'),
                    'name' => 'validate',
                    'required' => false,
                    'is_bool' => true,
                    'values' => array(
                        array(
                            'id' => 'validate_on',
                            'value' => 1,
                            'label' => $this->l('Yes')
                        ),
                        array(
                            'id' => 'validate_off',
                            'value' => 0,
                            'label' => $this->l('No')
                        )
                    ),
                ),
                'featured' => array(
                    'type' => 'switch',
                    'label' => $this->l('Featured:'),
                    'name' => 'featured',
                    'required' => false,
                    'is_bool' => true,
                    'values' => array(
                        array(
                            'id' => 'featured_on',
                            'value' => 1,
                            'label' => $this->l('Yes')
                        ),
                        array(
                            'id' => 'featured_off',
                            'value' => 0,
                            'label' => $this->l('No')
                        )
                    ),
                    'desc' => $this->l('Featured comments can be show on comments list firstly..')
                ),
                'home_featured' => array(
                    'type' => 'switch',
                    'label' => $this->l('Home featured:'),
                    'name' => 'home_featured',
                    'required' => false,
                    'is_bool' => true,
                    'values' => array(
                        array(
                            'id' => 'home_featured_on',
                            'value' => 1,
                            'label' => $this->l('Yes')
                        ),
                        array(
                            'id' => 'home_featured_off',
                            'value' => 0,
                            'label' => $this->l('No')
                        )
                    ),
                    'desc' => $this->l('Home featured comments will show on homepage, left / right columns and page footer.')
                ),
                array(
                    'type' => 'go_back_to_list',
                    'label' => '',
                    'name' => AdminController::$currentIndex.'&configure='.$this->name.($comment->id_parent ? '&view'.$this->name.'&id_st_product_comment='.$comment->id_parent : '').'&token='.Tools::getValue('token'),
                ),
            ),
            'buttons' => array(
                array(
                    'type' => 'submit',
                    'title' => $this->l('Save', array(), 'Admin.Actions'),
                    'icon' => 'process-icon-save',
                    'class'=> 'pull-right'
                ),
            ),
        );
        if ($id_parent) {
            unset($this->fields_form[0]['form']['input']['id_product']);
            unset($this->fields_form[0]['form']['input']['grade']);
            $this->fields_form[0]['form']['input'][] = array('type' => 'hidden', 'name' => 'id_parent');
        }
        if (!$comment->id) {
            $this->fields_form[0]['form']['input'][] = array('type' => 'hidden', 'name' => 'is_admin');
        }
        if ($comment->id_parent) {
            unset($this->fields_form[0]['form']['input']['id_product']);
            unset($this->fields_form[0]['form']['input']['grade']);
            unset($this->fields_form[0]['form']['input']['featured']);
            unset($this->fields_form[0]['form']['input']['home_featured']);
            $this->_html .= $this->getNavigate($comment->id ? $comment->id : $id_st_product_comment);
        }
        $helper = new HelperForm();
        $helper->module = $this;
        $helper->show_toolbar = false;
        $helper->id = $id_st_product_comment;
        $helper->table =  'st_product_comment';
        $helper->identifier = 'id_st_product_comment';
        $lang = new Language((int)Configuration::get('PS_LANG_DEFAULT'));
        $helper->default_form_language = $lang->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ? Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') : 0;
        
        $helper->submit_action = 'save'.$this->name;
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false).'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = array(
            'fields_value' => $this->getFieldsValueSt($comment),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id
        );
        return $helper;
    }
    public function renderCriterionForm()
    {
        $id_st_product_comment_criterion = Tools::getValue('id_st_product_comment_criterion');
        $criterion = new StProductCommentCriterionClass((int)$id_st_product_comment_criterion);
        $this->fields_form[0]['form'] = array(
            'legend' => array(
                'title' => $this->l('Add new criterion'),
                'icon' => 'icon-cogs',
            ),
            'input' => array(
                array(
                    'type' => 'hidden',
                    'name' => 'id_st_product_comment_criterion',
                ),
                array(
                    'type' => 'text',
                    'lang' => true,
                    'label' => $this->l('Criterion name'),
                    'name' => 'name',
                ),
                array(
                    'type' => 'switch',
                    'label' => $this->l('Active'),
                    'name' => 'active',
                    'values' => array(
                        array(
                            'id' => 'active_on',
                            'value' => 1,
                            'label' => $this->l('Enabled'),
                        ),
                        array(
                            'id' => 'active_off',
                            'value' => 0,
                            'label' => $this->l('Disabled'),
                        ),
                    ),
                ),
                array(
                    'type' => 'go_back_to_list',
                    'label' => '',
                    'name' => AdminController::$currentIndex.'&configure='.$this->name.'&set'.$this->name.'&token='.Tools::getValue('token').'#1',
                ),
            ),
            'submit' => array(
                'title' => $this->l('Save', array(), 'Admin.Actions'),
                'class' => 'btn btn-default pull-right',
                'name' => 'submitEditCriterion',
                ),
        );

        $helper = new HelperForm();
        $helper->module = $this;
        $helper->show_toolbar = false;
        $helper->table = $this->name;
        $lang = new Language((int) Configuration::get('PS_LANG_DEFAULT'));
        $helper->default_form_language = $lang->id;
        $helper->module = $this;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ? Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') : 0;
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'save'.$this->name.'criterion';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false).'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = array(
            'fields_value' => $this->getFieldsValueSt($criterion),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        );

        return $helper->generateForm($this->fields_form);
    }
    public function _prepareHook($ext = '')
    {
        $ext = $ext ? '_'.Tools::strtoupper($ext) : '';
        
        $slideshow      = Configuration::get($this->_prefix_st.'SLIDESHOW'.$ext);
        $s_speed        = Configuration::get($this->_prefix_st.'S_SPEED'.$ext);
        $a_speed        = Configuration::get($this->_prefix_st.'A_SPEED'.$ext);
        $pause_on_hover = Configuration::get($this->_prefix_st.'PAUSE_ON_HOVER'.$ext);
        $rewind_nav     = Configuration::get($this->_prefix_st.'REWIND_NAV'.$ext);
        $lazy_load      = Configuration::get($this->_prefix_st.'LAZY'.$ext);
        $move           = Configuration::get($this->_prefix_st.'MOVE'.$ext);
        $hide_mob       = Configuration::get($this->_prefix_st.'HIDE_MOB'.$ext);
        $aw_display     = Configuration::get($this->_prefix_st.'AW_DISPLAY'.$ext);
        
        $poster = Configuration::get($this->_prefix_st.'VIDEO_POSTER');
        if ($poster) {
            $this->fetchMediaServer($poster);
        }

        $this->smarty->assign(array(
            'pcomments'              => $this->getComments(trim($ext, '_')),
            'slider_slideshow'      => $slideshow,
            'slider_s_speed'        => $s_speed,
            'slider_a_speed'        => $a_speed,
            'slider_pause_on_hover' => $pause_on_hover,
            'rewind_nav'            => $rewind_nav,
            'lazy_load'             => $lazy_load,
            'slider_move'           => $move,
            'hide_mob'              => (int)$hide_mob,
            'aw_display'            => (int)$aw_display,
            'display_as_grid'       => Configuration::get($this->_prefix_st.'GRID'),
            'title_position'        => Configuration::get($this->_prefix_st.'TITLE_ALIGN'.$ext),
            'direction_nav'         => Configuration::get($this->_prefix_st.'DIRECTION_NAV'),
            'hide_direction_nav_on_mob' => Configuration::get($this->_prefix_st.'HIDE_DIRECTION_NAV_ON_MOB'),
            'control_nav'           => Configuration::get($this->_prefix_st.'CONTROL_NAV'),
            'hide_control_nav_on_mob' => Configuration::get($this->_prefix_st.'HIDE_CONTROL_NAV_ON_MOB'),
            'spacing_between'       => Configuration::get($this->_prefix_st.'SPACING_BETWEEN'),
            'display_pro_col'       => Configuration::get($this->_prefix_st.'DISPLAY_PRO_COL'),
            'content_width'       => Configuration::get($this->_prefix_st.'CONTENT_WIDTH'),
            

            'video_mpfour'          => Configuration::get($this->_prefix_st.'VIDEO_MPFOUR'),
            'video_webm'            => Configuration::get($this->_prefix_st.'VIDEO_WEBM'),
            'video_ogg'             => Configuration::get($this->_prefix_st.'VIDEO_OGG'),
            'video_loop'            => Configuration::get($this->_prefix_st.'VIDEO_LOOP'),
            'video_muted'           => Configuration::get($this->_prefix_st.'VIDEO_MUTED'),
            'video_poster'          => $poster,
            'video_v_offset'        => Configuration::get($this->_prefix_st.'VIDEO_V_OFFSET'),
        ));
        return true;
    }
    public function renderCommentExtra($hookName)
    {
        $id_product = (int)Tools::getValue('id_product');
        $is_quick_view = Tools::getValue('action')=='quickview' ? 1 : 0;
        if (!$this->isCached('stproductcomments_reviews.tpl', $this->stGetCacheId($hookName.$id_product.$is_quick_view))) {
            $average = StProductCommentClass::getAverageGrade($id_product);
            $nbComments = (int) StProductCommentClass::getCommentNumber($id_product);
            $this->smarty->assign(array(
                'average' => $average,
                'commentNbr' => $nbComments,
                'is_quick_view' => $is_quick_view,
                'pcomment_link' => $this->context->link->getModuleLink('stproductcomments', 'list', array('id_product'=>$id_product)),
            ));
        }
        return $this->fetch('module:stproductcomments/views/templates/hook/stproductcomments_reviews.tpl', $this->stGetCacheId($hookName.$id_product.$is_quick_view));
    }
    public function hookDisplayCustomerAccount($params)
    {
        return $this->display(__FILE__, 'my-account'.($this->_st_themes ? '_st' : '').'.tpl');
    }
    public function hookDisplayProductListReviews($params)
    {
        if (!isset($params['product'])) {
            return false;
        }
        $id_product = (int)$params['product']['id_product'];
        
        if (!$this->isCached('stproductcomments_reviews.tpl', $this->stGetCacheId($id_product))) {
            $comment = StProductCommentClass::getRatings($id_product);

            $this->smarty->assign(array(
                    'display_rating' => Configuration::get($this->_prefix_st.'DISPLAY_RATING'),
                    'display_as_link' => Configuration::get($this->_prefix_st.'DISPLAY_AS_LINK'),
                    'rating_align' => Configuration::get($this->_prefix_st.'RATING_ALIGN'),
                    'pro_posi' => Configuration::get($this->_prefix_st.'PRO_POSI'),
                    'total' => $comment ? $comment['total'] : 0,
                    'average' => $comment ? $comment['avg'] : 0,
                    'pc_id_product' => $id_product,
                ));
        }
        return $this->fetch('module:stproductcomments/views/templates/hook/pcomments_rating.tpl', $this->stGetCacheId($id_product));
    }
    private function _prepareComments($id_product)
    {
        if (!$id_product) {
            return false;
        }
        $averages = StProductCommentClass::getAveragesByProduct($id_product, $this->context->language->id);
        
        $averageTotal = 0;
        foreach ($averages as $average) {
            $averageTotal += (float)($average);
        }
        $averageTotal = count($averages) ? round($averageTotal / count($averages), 1) : 0;
        
        $criterions = StProductCommentCriterionClass::getCriterions($this->context->language->id, true);
        $criterions_arr = array();
        if ($criterions) {
            foreach ($criterions as $v) {
                $criterions_arr[$v['id_st_product_comment_criterion']] = $v['name'];
            }
        }
        $pro_tag_nbr = (int)Configuration::get($this->_prefix_st.'PRO_TAG_NBR');
        $biaoqian = StProductCommentClass::getTagByProduct($id_product, ($pro_tag_nbr ? $pro_tag_nbr : 12));

        $nbComments = (int) StProductCommentClass::getCommentNumber($id_product);
        // $product = $this->context->controller->getProduct();
        $write_flag = (int)Configuration::get($this->_prefix_st.'WRITE_FLAG');
        
        $this->smarty->assign(array(
            'write_flag'    => $write_flag,
            'logged'        => $this->context->customer->isLogged(true),
            'customer_name' => $this->context->customer->isLogged(true) ? $this->context->customer->firstname.' '.$this->context->customer->lastname : '',
            'id_module'     => $this->id,
            'id_product'    => $id_product,
            'upload_image'  => Configuration::get($this->_prefix_st.'UPLOAD_IMAGE'),
            'link'          => $this->context->link,
            'pcomment_link' => $this->context->link->getModuleLink('stproductcomments', 'list', array('id_product' => $id_product)),
            'current_url'   => $this->context->shop->getBaseURL(true, false).$_SERVER['REQUEST_URI'],
            'criterions'    => $criterions_arr,
            'criterions2'   => $criterions,
            'averages'      => $averages,
            'averageTotal'  => $averageTotal,
            'biaoqian'      => $biaoqian,
            'g_recaptcha'   => Configuration::get($this->_prefix_st.'GOOGLE_RECAPTCHA'),
            'g_recaptcha_key' => Configuration::get($this->_prefix_st.'RECAPTCHA_SITE_KEY'),
            'can_comment'   => StProductCommentClass::canComment($this->context->cookie->id_customer, $id_product),
            'is_bought'     => ($write_flag == 1) && StProductCommentClass::isBoughtProduct($this->context->cookie->id_customer, $id_product),
            'g_rich_snippets' => Configuration::get($this->_prefix_st.'GOOGLE_RICH_SNIPPETS'),
            'aw_display'    => Configuration::get($this->_prefix_st.'AW_DISPLAY'),
            'nbComments'    => $nbComments,
            'extra_info'    => Configuration::get($this->_prefix_st.'EXTRA_MESSAGE', $this->context->language->id),
            'form_layout'    => Configuration::get($this->_prefix_st.'FORM_LAYOUT'),
            'st_product_comment_tag_prefilled' => implode(',', StProductCommentClass::getTag($id_product)),
            'stproductcomments_url' => $this->context->link->getModuleLink('stproductcomments','default'),
        ));

        return true;
    }
    public function hookActionProductDelete($params)
    {
        $id_product = isset($params['id_product']) ? $params['id_product'] : 0;
        if ($id_product) {
            StProductCommentClass::deleteByIdProduct($id_product);
        }
    }
    public function hookDisplayFooterProduct($params)
    {
        $id_product = Tools::getValue('id_product');
        if (!$id_product) {
            return false;
        }

        $this->_prepareComments($id_product);
        $this->smarty->assign(array(
            'title_position'        => Configuration::get($this->_prefix_st.'PF_TITLE_ALIGN'),
        ));

        return $this->fetch('module:stproductcomments/views/templates/hook/pcomments_block.tpl');
    }
    public function hookDisplayProductExtraContent($params)
    {
        $extraContent = new ProductExtraContent();
        
        if (!isset($params['product'])) {
            return $extraContent;
        }
        $id_product = (int)$params['product']->id;
        
        $this->_prepareComments($id_product);
        $nbComments = (int) StProductCommentClass::getCommentNumber($id_product);

        $extraContent->setTitle(
            $this->l('Reviews').
            ($nbComments ? '('.$nbComments.')' : '')
        )->setContent(
            $this->fetch('module:stproductcomments/views/templates/hook/stproductcomments.tpl')
        );

        return array($extraContent);
    }
    public function hookDisplayHeader($params)
    {
        $this->context->controller->addCSS($this->_path.'views/css/stproductcomments.css', 'all');
        $this->context->controller->addCSS($this->_path.'views/font/fontello-codes.css');
        if (!Configuration::get($this->_prefix_st.'COMPATIBILITY_MODE')) {
            $this->context->controller->addCSS($this->_path.'views/css/stgeneral.css');
        }
        if (!Configuration::get($this->_prefix_st.'LOAD_SWIPER')) {
            $this->context->controller->addJS($this->_path.'views/js/swiper.jquery.min.js');
            $this->context->controller->addCSS($this->_path.'views/css/swiper.css', 'all');
        }
        if (!Configuration::get($this->_prefix_st.'LOAD_MAGNIFIC_POPUP')) {
            $this->context->controller->addJS($this->_path.'views/js/jquery.magnific-popup.min.js');
            $this->context->controller->addCSS($this->_path.'views/css/magnific-popup.css', 'all');
        }
        Media::addJsDef(array(
            'confirm_report_message' => $this->l('Are you sure report abuse ?'),
            'dictRemoveFile' => $this->l('Remove'),
            'st_pc_display_format' => $this->l('#left / #max Characters left.'),
            'st_pc_max' => Configuration::get($this->_prefix_st.'MAX_COMMENT') ? Configuration::get($this->_prefix_st.'MAX_COMMENT') : 500,
            'stproductcomments_controller_url' => $this->context->link->getModuleLink('stproductcomments', 'default', array('secure_key'=>$this->secure_key)),
            'st_pc_max_images' => Configuration::get($this->_prefix_st.'MAX_IMAGES') ? Configuration::get($this->_prefix_st.'MAX_IMAGES') : 6,
        ));
        if (isset($this->context->controller->module) && $this->context->controller->module->name == $this->name || Dispatcher::getInstance()->getController() == 'product') {
            $this->context->controller->registerJavascript('jquery-rating-pack', 'modules/'.$this->name.'/views/js/jquery.rating.pack.js', array('position' => 'bottom', 'priority' => 150));
            $this->context->controller->registerJavascript('jquery-tagmanager', 'modules/'.$this->name.'/views/js/tagmanager.js', array('position' => 'bottom', 'priority' => 150));
            $this->context->controller->registerJavascript('jquery-textareacounter', 'modules/'.$this->name.'/views/js/jquery.textareaCounter.js', array('position' => 'bottom', 'priority' => 150));
        }
        $this->context->controller->registerJavascript('modules-stproductcomments', 'modules/'.$this->name.'/views/js/stproductcomments.js', array('position' => 'bottom', 'priority' => 150));
        if (Configuration::get($this->_prefix_st.'GOOGLE_RECAPTCHA') && isset($this->context->controller->module) && $this->context->controller->module->name == $this->name || Dispatcher::getInstance()->getController() == 'product') {
            $this->context->controller->registerJavascript('google-recaptcha', 'https://www.google.com/recaptcha/api.js', array('position' => 'bottom', 'priority' => 9999, 'server' => 'remote'));
        }
        if (Configuration::get($this->_prefix_st.'UPLOAD_IMAGE') && isset($this->context->controller->module) && $this->context->controller->module->name == $this->name || Dispatcher::getInstance()->getController() == 'product') {
            $this->context->controller->registerJavascript('dropzone', 'modules/'.$this->name.'/views/js/dropzone.min.js', array('position' => 'bottom', 'priority' => 150));
        }
        
        $template = 'module:stproductcomments/views/templates/hook/header.tpl';
        if (!$this->isCached($template, $this->stGetCacheId('header'))) {
            $classname = $this->name.'_container';
            
            $custom_css = '';
            
            $group_css = '';
            if ($bg_color = Configuration::get($this->_prefix_st.'BG_COLOR')) {
                $group_css .= 'background-color:'.$bg_color.';';
            }
            if ($bg_img = Configuration::get($this->_prefix_st.'BG_IMG')) {
                $this->fetchMediaServer($bg_img);
                $group_css .= 'background-image: url('.$bg_img.');';
            } elseif ($bg_pattern = Configuration::get($this->_prefix_st.'BG_PATTERN')) {
                $img = _MODULE_DIR_.'stproductcomments/views/img/'.$bg_pattern.'.png';
                $img = $this->context->link->protocol_content.Tools::getMediaServer($img).$img;
                $group_css .= 'background-image: url('.$img.');background-repeat: repeat;';
            }
            if ($group_css) {
                $custom_css .= '.'.$classname.'.products_container{'.$group_css.'}';
            }
            /*if ($bg_img_v_offset = (int)Configuration::get($this->_prefix_st.'BG_IMG_V_OFFSET')) {
                $custom_css .= '.'.$classname.'.products_container{background-position:center -'.$bg_img_v_offset.'px;}';
            }*/
    
            if ($top_padding = (int)Configuration::get($this->_prefix_st.'TOP_PADDING')) {
                $custom_css .= '.'.$classname.'.products_container  .products_section{padding-top:'.$top_padding.'px;}';
            }
            if ($bottom_padding = (int)Configuration::get($this->_prefix_st.'BOTTOM_PADDING')) {
                $custom_css .= '.'.$classname.'.products_container  .products_section{padding-bottom:'.$bottom_padding.'px;}';
            }
            $top_margin = Configuration::get($this->_prefix_st.'TOP_MARGIN');
            if ($top_margin || $top_margin===0 || $top_margin==='0') {
                $custom_css .= '.'.$classname.'.products_container{margin-top:'.$top_margin.'px;}';
            }
            $bottom_margin = Configuration::get($this->_prefix_st.'BOTTOM_MARGIN');
            if ($bottom_margin || $bottom_margin===0 || $bottom_margin==='0') {
                $custom_css .= '.'.$classname.'.products_container{margin-bottom:'.$bottom_margin.'px;}';
            }
            if ($title_font_size = (int)Configuration::get($this->_prefix_st.'TITLE_FONT_SIZE')) {
                $custom_css .= '.'.$classname.'.products_container .title_block_inner{font-size:'.$title_font_size.'px;}';
            }
            if ($title_color = Configuration::get($this->_prefix_st.'TITLE_COLOR')) {
                $custom_css .= '.'.$classname.'.products_container .title_block_inner{color:'.$title_color.';}';
            }
            if ($title_hover_color = Configuration::get($this->_prefix_st.'TITLE_HOVER_COLOR')) {
                $custom_css .= '.'.$classname.'.products_container .title_block_inner:hover{color:'.$title_hover_color.';}';
            }
            $heading_bottom_border = Configuration::get($this->_prefix_st.'TITLE_BOTTOM_BORDER');
            if ($heading_bottom_border || $heading_bottom_border===0 || $heading_bottom_border==='0') {
                $custom_css .= '.'.$classname.'.products_container .title_style_0,.'.$classname.'.products_container .title_style_0 .title_block_inner{border-bottom-width:'.$heading_bottom_border.'px;}.'.$classname.'.products_container .title_style_0 .title_block_inner{margin-bottom:-'.$heading_bottom_border.'px;}';
            }
            
            if (Configuration::get($this->_prefix_st.'TITLE_BOTTOM_BORDER_COLOR')) {
                $custom_css .='.'.$classname.'.products_container .title_style_0{border-bottom-color: '.Configuration::get($this->_prefix_st.'TITLE_BOTTOM_BORDER_COLOR').';}';
            }
            if (Configuration::get($this->_prefix_st.'TITLE_BOTTOM_BORDER_COLOR_H')) {
                $custom_css .='.'.$classname.'.products_container .title_style_0 .title_block_inner{border-color: '.Configuration::get($this->_prefix_st.'TITLE_BOTTOM_BORDER_COLOR_H').';}';
            }
            if ($text_color = Configuration::get($this->_prefix_st.'TEXT_COLOR')) {
                $custom_css .= '.'.$classname.'.products_container .pcomments_item,.'.$classname.'.products_container .pcomments_item .pc_s_title a{color:'.$text_color.';}';
            }
            if ($link_color = Configuration::get($this->_prefix_st.'LINK_COLOR')) {
                $custom_css .= '.'.$classname.'.products_container .pcomments_item .pc_s_title a{color:'.$link_color.';}';
            }
            if ($link_hover_color = Configuration::get($this->_prefix_st.'LINK_HOVER_COLOR')) {
                $custom_css .= '.'.$classname.'.products_container .pcomments_item .pc_s_title a:hover{color:'.$link_hover_color.';}';
            }
            if ($text_color_col = Configuration::get($this->_prefix_st.'TEXT_COLOR_COL')) {
                $custom_css .= '.'.$classname.'.products_container_column .pcomments_item,.'.$classname.'.products_container_column .pcomments_item .pc_s_title a{color:'.$text_color_col.';}';
            }
            if ($link_color_col = Configuration::get($this->_prefix_st.'LINK_COLOR_COL')) {
                $custom_css .= '.'.$classname.'.products_container_column .pcomments_item .pc_s_title a{color:'.$link_color_col.';}';
            }
            if ($link_hover_color_col = Configuration::get($this->_prefix_st.'LINK_HOVER_COLOR_COL')) {
                $custom_css .= '.'.$classname.'.products_container_column .pcomments_item .pc_s_title a:hover{color:'.$link_hover_color_col.';}';
            }
            if ($text_color_fot = Configuration::get($this->_prefix_st.'TEXT_COLOR_FOT')) {
                $custom_css .= '.pcomment_footer .pcomments_item, .pcomment_footer .pcomments_item .pc_s_title a{color:'.$text_color_fot.';}';
            }
            if ($link_color_fot = Configuration::get($this->_prefix_st.'LINK_COLOR_FOT')) {
                $custom_css .= '.pcomment_footer .pcomments_item .pc_s_title a{color:'.$link_color_fot.';}';
            }
            if ($link_hover_color_fot = Configuration::get($this->_prefix_st.'LINK_HOVER_COLOR_FOT')) {
                $custom_css .= '.pcomment_footer .pcomments_item .pc_s_title a:hover{color:'.$link_hover_color_fot.';}';
            }
            //.block is used to make bg take effect, .swiper-button-lr, .swiper-button-tr hui fu gai
            if ($direction_color = Configuration::get($this->_prefix_st.'DIRECTION_COLOR')) {
                $custom_css .= '.'.$classname.'.block .products_section .swiper-button{color:'.$direction_color.';}';
            }
            if ($direction_color_hover = Configuration::get($this->_prefix_st.'DIRECTION_COLOR_HOVER')) {
                $custom_css .= '.'.$classname.'.block .products_section .swiper-button:hover{color:'.$direction_color_hover.';}';
            }
            if ($direction_color_disabled = Configuration::get($this->_prefix_st.'DIRECTION_COLOR_DISABLED')) {
                $custom_css .= '.'.$classname.'.block .products_section .swiper-button.swiper-button-disabled, .'.$classname.' .products_section .swiper-button.swiper-button-disabled:hover{color:'.$direction_color_disabled.';}';
            }
            if ($direction_bg = Configuration::get($this->_prefix_st.'DIRECTION_BG')) {
                $custom_css .= '.'.$classname.' .products_section .swiper-button{background-color:'.$direction_bg.';}';
            }
            if ($direction_hover_bg = Configuration::get($this->_prefix_st.'DIRECTION_HOVER_BG')) {
                $custom_css .= '.'.$classname.' .products_section .swiper-button:hover{background-color:'.$direction_hover_bg.';}';
            }
            if ($direction_disabled_bg = Configuration::get($this->_prefix_st.'DIRECTION_DISABLED_BG')) {
                $custom_css .= '.'.$classname.' .products_section .swiper-button.swiper-button-disabled, .'.$classname.' .products_section .swiper-button.swiper-button-disabled:hover{background-color:'.$direction_disabled_bg.';}';
            } else {
                $custom_css .= '.'.$classname.' .products_section .swiper-button.swiper-button-disabled, .'.$classname.' .products_section .swiper-button.swiper-button-disabled:hover{background-color:transplanted;}';
            }
    
            if ($pag_nav_bg = Configuration::get($this->_prefix_st.'PAG_NAV_BG')) {
                $custom_css .= '.'.$classname.' .swiper-pagination-bullet,.'.$classname.' .swiper-pagination-progress{background-color:'.$pag_nav_bg.';}';
                $custom_css .= '.'.$classname.' .swiper-pagination-st-round .swiper-pagination-bullet{background-color:transparent;border-color:'.$pag_nav_bg.';}';
                $custom_css .= '.'.$classname.' .swiper-pagination-st-round .swiper-pagination-bullet span{background-color:'.$pag_nav_bg.';}';
            }
            if ($pag_nav_bg_hover = Configuration::get($this->_prefix_st.'PAG_NAV_BG_HOVER')) {
                $custom_css .= '.'.$classname.' .swiper-pagination-bullet-active, .'.$classname.' .swiper-pagination-progress .swiper-pagination-progressbar{background-color:'.$pag_nav_bg_hover.';}';
                $custom_css .= '.'.$classname.' .swiper-pagination-st-round .swiper-pagination-bullet.swiper-pagination-bullet-active{background-color:'.$pag_nav_bg_hover.';border-color:'.$pag_nav_bg_hover.';}';
                $custom_css .= '.'.$classname.' .swiper-pagination-st-round .swiper-pagination-bullet.swiper-pagination-bullet-active span{background-color:'.$pag_nav_bg_hover.';}';
            }

            if ($star_highlight_color = Configuration::get($this->_prefix_st.'STAR_HIGHLIGHT_COLOR')) {
                $custom_css .= '.pc_grade.icon_btn.light{color:'.$star_highlight_color.';}';
            }
            if ($star_color = Configuration::get($this->_prefix_st.'STAR_COLOR')) {
                $custom_css .= '.pc_grade.icon_btn{color:'.$star_color.';}';
            }
            if ($star_icon = Configuration::get($this->_prefix_st.'STAR_ICON')) {
                $custom_css .= '.pc_grade.icon_btn:after, div.star:after{ content: "\\'.$star_icon.'"; }';
            }
            if ($star_size = Configuration::get($this->_prefix_st.'STAR_SIZE')) {
                $custom_css .= '.pc_grade.icon_btn{font-size:'.$star_size.'px;}';
            }
            if ($this->context->shop->theme->getName()=='classic' && !Configuration::get($this->_prefix_st.'CLASSIC_THEME')) {
                $custom_css .= '.product-description .rating_box{padding:6px;position: absolute;top:-30px;left:0;right:0;}.thumbnail-container:focus .rating_box, .thumbnail-container:hover .rating_box{display:none;}';
            }

            if ($pf_title_font_size = (int)Configuration::get($this->_prefix_st.'PF_TITLE_FONT_SIZE')) {
                $custom_css .= '.pcomments_block .title_block_inner{font-size:'.$pf_title_font_size.'px;}';
            }
            if ($pf_title_color = Configuration::get($this->_prefix_st.'PF_TITLE_COLOR')) {
                $custom_css .= '.pcomments_block .title_block_inner{color:'.$pf_title_color.';}';
            }
            if ($pf_title_hover_color = Configuration::get($this->_prefix_st.'PF_TITLE_HOVER_COLOR')) {
                $custom_css .= '.pcomments_block .title_block_inner:hover{color:'.$pf_title_hover_color.';}';
            }
            $pf_heading_bottom_border = Configuration::get($this->_prefix_st.'PF_TITLE_BOTTOM_BORDER');
            if ($pf_heading_bottom_border || $pf_heading_bottom_border===0 || $pf_heading_bottom_border==='0') {
                $custom_css .= '.pcomments_block .title_style_0,.pcomments_block .title_style_0 .title_block_inner{border-bottom-width:'.$pf_heading_bottom_border.'px;}.pcomments_block .title_style_0 .title_block_inner{margin-bottom:-'.$pf_heading_bottom_border.'px;}';
            }
            
            if (Configuration::get($this->_prefix_st.'PF_TITLE_BOTTOM_BORDER_COLOR')) {
                $custom_css .='.pcomments_block .title_style_0{border-bottom-color: '.Configuration::get($this->_prefix_st.'PF_TITLE_BOTTOM_BORDER_COLOR').';}';
            }
            if (Configuration::get($this->_prefix_st.'PF_TITLE_BOTTOM_BORDER_COLOR_H')) {
                $custom_css .='.pcomments_block .title_style_0 .title_block_inner{border-color: '.Configuration::get($this->_prefix_st.'PF_TITLE_BOTTOM_BORDER_COLOR_H').';}';
            }

            if ($title_font_size_col = (int)Configuration::get($this->_prefix_st.'TITLE_FONT_SIZE_COL')) {
                $custom_css .= '.'.$classname.'.products_container_column .title_block_inner{font-size:'.$title_font_size_col.'px;}';
            }
            if ($title_color_col = Configuration::get($this->_prefix_st.'TITLE_COLOR_COL')) {
                $custom_css .= '.'.$classname.'.products_container_column .title_block_inner{color:'.$title_color_col.';}';
            }
            if ($title_hover_color_col = Configuration::get($this->_prefix_st.'TITLE_HOVER_COLOR_COL')) {
                $custom_css .= '.'.$classname.'.products_container_column .title_block_inner:hover{color:'.$title_hover_color_col.';}';
            }
            /*$heading_bottom_border_col = Configuration::get($this->_prefix_st.'TITLE_BOTTOM_BORDER_COL');
            if ($heading_bottom_border_col || $heading_bottom_border_col===0 || $heading_bottom_border_col==='0') {
                $custom_css .= '.'.$classname.'.products_container_column .title_style_0, .'.$classname.'.products_container_column .title_style_0 .title_block_inner{border-bottom-width:'.$heading_bottom_border_col.'px;}.'.$classname.'.products_container_column .title_style_0 .title_block_inner{margin-bottom:-'.$heading_bottom_border_col.'px;}';
            }
            
            if (Configuration::get($this->_prefix_st.'TITLE_BOTTOM_BORDER_COLOR_COL')) {
                $custom_css .='.'.$classname.'.products_container_column .title_style_0{border-bottom-color: '.Configuration::get($this->_prefix_st.'TITLE_BOTTOM_BORDER_COLOR_COL').';}';
            }
            if (Configuration::get($this->_prefix_st.'TITLE_BOTTOM_BORDER_COLOR_H_COL')) {
                $custom_css .='.'.$classname.'.products_container_column .title_style_0 .title_block_inner{border-color: '.Configuration::get($this->_prefix_st.'TITLE_BOTTOM_BORDER_COLOR_H_COL').';}';
            }*/

            if ($title_color_fot = Configuration::get($this->_prefix_st.'TITLE_COLOR_FOT')) {
                $custom_css .= '.pcomment_footer_title a{color:'.$title_color_fot.';}';
            }
            if ($title_hover_color_fot = Configuration::get($this->_prefix_st.'TITLE_HOVER_COLOR_FOT')) {
                $custom_css .= '.pcomment_footer_title a:hover{color:'.$title_hover_color_fot.';}';
            }

            
            if ($custom_css) {
                $this->smarty->assign('custom_css', preg_replace('/\s\s+/', ' ', $custom_css));
            }
        }
        return $this->fetch($template, $this->stGetCacheId('header'));
    }
    public function hookDisplayWrapperTop($params)
    {
        if (Dispatcher::getInstance()->getController()!='index') {
            return false;
        }

        return $this->hookDisplayHome($params, __FUNCTION__, 2);
    }
    public function hookDisplayWrapperBottom($params)
    {
        if (Dispatcher::getInstance()->getController()!='index') {
            return false;
        }

        return $this->hookDisplayHome($params, __FUNCTION__, 2);
    }
    public function hookDisplayNavFullWidth($params)
    {
        if (dispatcher::geTinstance()->geTcontroller()!='index') {
            return false;
        }

        return $this->hookDisplayHome($params, __FUNCTION__, 2);
    }
    public function hookDisplayStFullWidthBottom($params)
    {
        if (dispatcher::geTinstance()->geTcontroller()!='index') {
            return false;
        }

        return $this->hookDisplayHome($params, __FUNCTION__, 2);
    }
    public function hookDisplayContentWrapperTop($params)
    {
        return $this->hookDisplayHome($params, __FUNCTION__);
    }
    public function hookDisplayContentWrapperBottom($params)
    {
        return $this->hookDisplayHome($params, __FUNCTION__);
    }
    public function hookDisplayFullWidthBottom($params)
    {
        if (Dispatcher::getInstance()->getController()!='index') {
            return false;
        }

        return $this->hookDisplayHome($params, __FUNCTION__, 2);
    }
    public function hookDisplayLeftColumnProduct($params)
    {
        return $this->hookDisplayLeftColumn($params, __FUNCTION__);
    }
    public function hookDisplayRightColumnProduct($params)
    {
        return $this->hookDisplayLeftColumn($params, __FUNCTION__);
    }
    
    //abstract public function hookDisplayLeftColumn($params);
    public function hookDisplayRightColumn($params)
    {
        return $this->hookDisplayLeftColumn($params, __FUNCTION__);
    }
    
    //abstract function hookDisplayFooter($params);
    public function hookDisplayFooterAfter($params)
    {
        return $this->hookDisplayFooter($params, __FUNCTION__);
    }
    public function hookDisplayFooterBefore($params)
    {
        return $this->hookDisplayFooter($params, __FUNCTION__);
    }
    public function getComments($ext = '')
    {
        $ext = $ext ? '_'.Tools::strtoupper($ext) : '';
        $nbr = Configuration::get($this->_prefix_st.'NBR'.$ext);
        $order_by = 'date_add';
        $order_way = 'DESC';
        $soby = (int)Configuration::get($this->_prefix_st.'SOBY'.$ext);
        switch ($soby) {
            case 0:
                $order_by = 'featured';
                $order_way = 'DESC';
                break;
            case 1:
                $order_by = 'total_useful';
                $order_way = 'DESC';
                break;
            case 2:
                $order_by = 'date_add';
                $order_way = 'DESC';
                break;
            case 3:
                $order_by = 'date_add';
                $order_way = 'ASC';
                break;
            case 4:
                $order_by = 'total_advice';
                $order_way = 'DESC';
                break;
        }
        $comments = StProductCommentClass::getListComments($nbr, 1, null, null, false, $order_by, $order_way, false, 0, 0, 1);

        if ($comments) {
            $assembler = new ProductAssembler($this->context);
            $presenterFactory = new ProductPresenterFactory($this->context);
            $presentationSettings = $presenterFactory->getPresentationSettings();
            $presenter = new ProductListingPresenter(
                new ImageRetriever(
                    $this->context->link
                ),
                $this->context->link,
                new PriceFormatter(),
                new ProductColorsRetriever(),
                $this->context->getTranslator()
            );
            
            foreach ($comments as &$value) {
                $value['timeago'] = Configuration::get($this->_prefix_st.'DISPLAY_DATE') ? false : StProductCommentClass::Timeago($value['date_add']);
                if ($value['id_product']) {
                    $value['product'] = $presenter->present(
                        $presentationSettings,
                        $assembler->assembleProduct(array(
                            'id_product' => $value['id_product'],
                        )),
                        $this->context->language
                    );
                } else {
                    $value['product'] = array();
                }
                $array = explode(' ', $value['customer_name']);
                $custom_name = '';
                $custom_name_ds = Configuration::get($this->_prefix_st.'CUSTOMER_NAME');
                if (!$custom_name_ds) {
                    foreach($array AS $i => $v) {
                        if (!$i) {
                            $v = substr($v, 0, 2).str_repeat('*', strlen($v)-2);    
                        } elseif ($i == count($array)-1) {
                            $v = str_repeat('*', strlen($v)-2).substr($v, -2);
                        } else {
                            $v = str_repeat('*', strlen($v));
                        }
                        $custom_name .= $v. ' ';
                    }
                    $value['customer_name'] = trim($custom_name);
                } elseif ($custom_name_ds == 2) {
                    $custom_name = array_shift($array);
                    $value['customer_name'] = trim($custom_name);
                }
            }
        }
        return $comments;
    }
    public function hookDisplayHome($params, $func = '', $flag = 0)
    {
        $hook_hash = $this->getHookHash($func ? $func : __FUNCTION__);
        $template = 'module:stproductcomments/views/templates/hook/pcomments_slider.tpl';
        if (!$this->isCached($template, $this->stGetCacheId($hook_hash))) {
            $this->_prepareHook();
            $this->smarty->assign(array(
                'column_slider'    => false,
                'homeverybottom'   => ($flag==2 ? true : false),
                'hook_hash'        => $hook_hash,
                'pro_per_fw'       => (int)Configuration::get($this->_prefix_stsn.'PRO_PER_FW'),
                'pro_per_xxl'      => (int)Configuration::get($this->_prefix_stsn.'PRO_PER_XXL'),
                'pro_per_xl'       => (int)Configuration::get($this->_prefix_stsn.'PRO_PER_XL'),
                'pro_per_lg'       => (int)Configuration::get($this->_prefix_stsn.'PRO_PER_LG'),
                'pro_per_md'       => (int)Configuration::get($this->_prefix_stsn.'PRO_PER_MD'),
                'pro_per_sm'       => (int)Configuration::get($this->_prefix_stsn.'PRO_PER_SM'),
                'pro_per_xs'       => (int)Configuration::get($this->_prefix_stsn.'PRO_PER_XS'),

                'has_background_img'    => ((int)Configuration::get($this->_prefix_st.'BG_PATTERN') || Configuration::get($this->_prefix_st.'BG_IMG')) ? 1 : 0,
                'speed'                 => Configuration::get($this->_prefix_st.'SPEED'),
                'bg_img_v_offset'       => (int)Configuration::get($this->_prefix_st.'BG_IMG_V_OFFSET'),
            ));
        }
        return $this->fetch($template, $this->stGetCacheId($hook_hash));
    }
    public function hookDisplayFooter($params, $func = '')
    {
        $hook_hash = $this->getHookHash($func ? $func : __FUNCTION__);
        $template = 'module:stproductcomments/views/templates/hook/pcomments_footer'.($this->_st_themes ? '_st' : '').'.tpl';
        if (!$this->isCached($template, $this->stGetCacheId($hook_hash))) {
            $this->smarty->assign(array(
                'pcomments'          => $this->getComments('fot'),
                'hide_mob'          => Configuration::get($this->_prefix_st.'HIDE_MOB_FOT'),
                'aw_display'        => Configuration::get($this->_prefix_st.'AW_DISPLAY_FOT'),
                'footer_wide'       => Configuration::get($this->_prefix_st.'FOOTER_WIDE'),
                'hook_hash'         => $hook_hash,
            ));
        }
        return $this->fetch($template, $this->stGetCacheId($hook_hash));
    }
    public function hookDisplayLeftColumn($params, $func = '')
    {
        $hook_hash = $this->getHookHash($func ? $func : __FUNCTION__);
        $template = 'module:stproductcomments/views/templates/hook/pcomments_slider.tpl';
        if (!$this->isCached($template, $this->stGetCacheId($hook_hash))) {
            $this->_prepareHook('col');
            
            $this->smarty->assign(array(
                'column_slider'         => true,
                'homeverybottom'        => false,
                'hook_hash'             => $hook_hash,
            ));
        }
        return $this->fetch($template, $this->stGetCacheId($hook_hash));
    }
    public function hookDisplayTop($params)
    {
        $template = 'module:stproductcomments/views/templates/hook/gcr_badge.tpl';
        $this->assignGcrBadge(true);
        return $this->fetch($template);
    }
    public function hookDisplayOrderConfirmation($params)
    {
        if (!isset($params['order']) || !$params['order']->id) {
            return;
        }
        // Only enable for valid states
        $id_order_state = $params['order']->current_state;
        if (!Configuration::get($this->_prefix_st.strtoupper('gcr_order_state_'.$id_order_state))) {
            return;
        }

        $presenter = new PrestaShop\PrestaShop\Adapter\Order\OrderPresenter();
        $presentedOrder = $presenter->present($params['order']);
        $processing_days = (int)Configuration::get($this->_prefix_st.strtoupper('gcr_process_days'));
        $delivery_days = (int)Configuration::get($this->_prefix_st.strtoupper('carrier_est_').$params['order']->id_carrier);
        // Payment date
        $pay_date = $presentedOrder['history']['current']['date_add'];
        $estimated_date = date('Y-m-d', strtotime('+ '.($processing_days+$delivery_days).' day', strtotime($pay_date)));
        $id_country = $presentedOrder['addresses']['delivery']['id_country'];
        $country_code = Country::getIsoById($id_country);
        $this->smarty->assign(array(
            'gcr_merchant_id'    => Configuration::get($this->_prefix_st.strtoupper('gcr_merchant_id')),
            'gcr_optin_style'    => Configuration::get($this->_prefix_st.strtoupper('gcr_opt_in'), 'CENTER_DIALOG'),
            'gcr_locale'         => $this->getGcrlang(),
            'order_id'           => $params['order']->id,
            'email'              => $this->context->customer->email,
            'country_code'       => $country_code,
            'est_date'           => $estimated_date,
        ));
        return $this->fetch('module:stproductcomments/views/templates/hook/gcr_optin.tpl');
    }
    public function hookDisplayBeforeBodyClosingTag($params)
    {
        $template = 'module:stproductcomments/views/templates/hook/sign_in.tpl';
        $this->assignGcrBadge();
        return $this->fetch($template);
    }
    private function assignGcrBadge($show_inline = false)
    {
        $page = Dispatcher::getInstance()->getController();
        
        $this->smarty->assign(array(
            'gcr_enable'         => Configuration::get($this->_prefix_st.strtoupper('gcreview')),
            'gcr_merchant_id'    => Configuration::get($this->_prefix_st.strtoupper('gcr_merchant_id')),
            'gcr_badge'          => Configuration::get($this->_prefix_st.strtoupper('gcr_badge')),
            'gcr_badge_pos'      => Configuration::get($this->_prefix_st.strtoupper('gcr_badge_pos'), 'BOTTOM_LEFT'),
            'gcr_badge_page'     => Configuration::get($this->_prefix_st.strtoupper('gcr_badge_on_'.$page)),
            'gcr_locale'         => $this->getGcrlang(),
            'show_inline'        => $show_inline,
        ));
    }
    private function getGcrlang()
    {
        $supported_langs = array('af','ar-AE','cs','da','de','en_AU','en_GB','en_US','es','es-419','fil','fr','ga','id','it','ja','ms','nl','no','pl','pt_BR','pt_PT','ru','sv','tr','zh-CN','zh-TW');
        $locale = $this->context->language->locale;
        return in_array($locale, $supported_langs) ? $locale : 'en_US';
    }
    public function getCommentNumber($id_product, $link_rewrite = null)
    {
        if (!$id_product || !Validate::isUnsignedInt($id_product)) {
            return false;
        }
        if (!StProductCommentClass::acceptComment($id_product)) {
            return false;
        }
        
        $moderate = (int)Configuration::get($this->_prefix_st.'MODERATE');
        $comment_number = (int)StProductCommentClass::countComments($id_product, $this->context->shop->id, $moderate);
        $this->smarty->assign(array(
            'comment_number'=>$comment_number,
            'id_product' => $id_product,
        ));
        if ($link_rewrite) {
            $this->smarty->assign('link_rewrite', $link_rewrite);
        }
            
        return $this->display(__FILE__, 'comment_number.tpl');
    }
    private function getConfigFieldsValues()
    {
        $fields_value = array();
        foreach ($this->getFormFieldsDefault() as $k => $v) {
            $fields_value[$k] = Configuration::get($this->_prefix_st.Tools::strtoupper($k));
        }
        foreach ($this->fields_default_stsn as $k => $v) {
            $fields_value[$k] = Configuration::get($this->_prefix_stsn.Tools::strtoupper($k));
        }
        foreach ($this->_hooks as $key => $values) {
            if (!$key) {
                continue;
            }
            foreach ($values as $value) {
                $fields_value[$key.'_'.$value['id']] = 0;
                if ($id_hook = Hook::getIdByName($value['id'])) {
                    if (Hook::getModulesFromHook($id_hook, $this->id)) {
                        $fields_value[$key.'_'.$value['id']] = 1;
                    }
                }
            }
        }
        foreach($this->order_states AS $value) {
            if(Configuration::get($this->_prefix_st.strtoupper('order_state_'.$value['id']))) {
                $fields_value['order_state_'.$value['id']] = 1;
            }
            if(Configuration::get($this->_prefix_st.strtoupper('gcr_order_state_'.$value['id']))) {
                $fields_value['gcr_order_state_'.$value['id']] = 1;
            }
        }
        foreach($this->_pages AS $value) {
            if(Configuration::get($this->_prefix_st.strtoupper('gcr_badge_on_'.$value['id']))) {
                $fields_value['gcr_badge_on_'.$value['id']] = 1;
            }
        }
        $languages = Language::getLanguages(false);
        foreach($this->lang_field as $field) {
            $fields_value[$field] = array();
        }
        foreach ($languages as $language) {
            foreach($this->lang_field as $field) {
                $fields_value[$field][$language['id_lang']] = Configuration::get($this->_prefix_st.strtoupper($field), $language['id_lang']);
            }
        }
        $fields_value['csv_lang'] = '';
        $fields_value['cron_url'] = $this->context->link->getModuleLink('stproductcomments', 'cron', array('token' => Tools::getAdminToken($this->name._COOKIE_IV_), 'id_shop' => $this->context->shop->id, 'id_lang' => $this->context->language->id));
        return $fields_value;
    }
    public function renderWidget($hookName = null, array $configuration = array())
    {
        $this->initHookArray();
        $in_product_page = false;
        foreach ($this->_hooks as $sub_hooks) {
            foreach ($sub_hooks as $v) {
                if (Tools::strtolower($v['id'])==Tools::strtolower($hookName)) {
                    $in_product_page = isset($v['in_product_page']);
                    break 2;
                }
            }
        }
        if ($in_product_page) {
            return $this->renderCommentExtra($hookName);
        } else {
            return false;
        }
    }
    public function getWidgetVariables($hookName = null, array $configuration = array())
    {
        return false;
    }
    /**
     * Return the list of fields value
     *
     * @param object $obj Object
     * @return array
     */
    public function getFieldsValueSt($obj, $fields_form = "fields_form")
    {
        foreach ($this->$fields_form as $fieldset) {
            if (isset($fieldset['form']['input'])) {
                foreach ($fieldset['form']['input'] as $input) {
                    if (!isset($this->fields_value[$input['name']])) {
                        if (isset($input['type']) && $input['type'] == 'shop') {
                            if ($obj->id) {
                                $result = Shop::getShopById((int)$obj->id, $this->identifier, $this->table);
                                foreach ($result as $row) {
                                    $this->fields_value['shop'][$row['id_'.$input['type']]][] = $row['id_shop'];
                                }
                            }
                        } elseif (isset($input['lang']) && $input['lang']) {
                            foreach (Language::getLanguages(false) as $language) {
                                $fieldValue = $this->getFieldValueSt($obj, $input['name'], $language['id_lang']);
                                if (empty($fieldValue)) {
                                    if (isset($input['default_value']) && is_array($input['default_value']) && isset($input['default_value'][$language['id_lang']])) {
                                        $fieldValue = $input['default_value'][$language['id_lang']];
                                    } elseif (isset($input['default_value'])) {
                                        $fieldValue = $input['default_value'];
                                    }
                                }
                                $this->fields_value[$input['name']][$language['id_lang']] = $fieldValue;
                            }
                        } else {
                            $fieldValue = $this->getFieldValueSt($obj, $input['name']);
                            if ($fieldValue===false && isset($input['default_value'])) {
                                $fieldValue = $input['default_value'];
                            }
                            $this->fields_value[$input['name']] = $fieldValue;
                        }
                    }
                }
            }
        }
        return $this->fields_value;
    }
    
    /**
     * Return field value if possible (both classical and multilingual fields)
     *
     * Case 1 : Return value if present in $_POST / $_GET
     * Case 2 : Return object value
     *
     * @param object $obj Object
     * @param string $key Field name
     * @param integer $id_lang Language id (optional)
     * @return string
     */
    public function getFieldValueSt($obj, $key, $id_lang = null)
    {
        if ($id_lang) {
            $default_value = ($obj->id && isset($obj->{$key}[$id_lang])) ? $obj->{$key}[$id_lang] : false;
        } else {
            $default_value = isset($obj->{$key}) ? $obj->{$key} : false;
        }

        return Tools::getValue($key.($id_lang ? '_'.$id_lang : ''), $default_value);
    }
    protected function stGetCacheId($key, $name = null)
    {
        $cache_id = parent::getCacheId($name);
        return $cache_id.'_'.$key;
    }
    public function getFormFields()
    {
        require_once(dirname(__FILE__).'/formFields.php');
        return getFormFields($this);
    }
    public function getFormFieldsDefault()
    {
        $default = array();
        foreach ($this->getFormFields() as $key => $value) {
            if ($key == 'hook') {
                continue;
            }
            foreach ($value as $k => $v) {
                if (!$k || !is_array($v)) {
                    continue;
                }
                if ($v['type'] == 'html' || $v['type'] == 'file') {
                    continue;
                }
                $default[$k] = isset($v['default_value']) ? $v['default_value'] : '';
            }
        }
        return $default;
    }
    public function getPatternsArray()
    {
        $arr = array();
        for ($i = 1; $i <= 28; $i++) {
            $arr[] = array('id'=>$i,'name'=>$i);
        }
        return $arr;
    }
    protected function stUploadImage($item)
    {
        $result = array(
            'error' => array(),
            'image' => '',
            'thumb' => '',
        );
        if (isset($_FILES[$item]) && isset($_FILES[$item]['tmp_name']) && !empty($_FILES[$item]['tmp_name'])) {
            $type = Tools::strtolower(Tools::substr(strrchr($_FILES[$item]['name'], '.'), 1));
            $name = str_replace(strrchr($_FILES[$item]['name'], '.'), '', $_FILES[$item]['name']);
            $imagesize = array();
            $imagesize = @getimagesize($_FILES[$item]['tmp_name']);
            if (!empty($imagesize) &&
                in_array(Tools::strtolower(Tools::substr(strrchr($imagesize['mime'], '/'), 1)), array('jpg', 'gif', 'jpeg', 'png')) &&
                in_array($type, array('jpg', 'gif', 'jpeg', 'png'))) {
                $temp_name = tempnam(_PS_TMP_IMG_DIR_, 'PS');
                $salt = $name ? Tools::str2url($name) : sha1(microtime());
                $c_name = $salt;
                $c_name_thumb = $c_name.'-thumb';
                if ($upload_error = ImageManager::validateUpload($_FILES[$item])) {
                    $result['error'][] = $upload_error;
                } elseif (!$temp_name || !move_uploaded_file($_FILES[$item]['tmp_name'], $temp_name)) {
                    $result['error'][] = $this->displayError($this->l('An error occurred during the image upload.'));
                } else {
                    $infos = getimagesize($temp_name);
                    $ratio_y = 72;
                    $ratio_x = $infos[0] / ($infos[1] / $ratio_y);
                    if (!ImageManager::resize($temp_name, _PS_UPLOAD_DIR_.$this->name.'/'.$c_name.'.'.$type, null, null, $type) || !ImageManager::resize($temp_name, _PS_UPLOAD_DIR_.$this->name.'/'.$c_name_thumb.'.'.$type, $ratio_x, $ratio_y, $type)) {
                        $result['error'][] = $this->displayError($this->l('An error occurred during the image upload.'));
                    }
                }
                if (isset($temp_name)) {
                    @unlink($temp_name);
                }
                if (!count($result['error'])) {
                    $result['image'] = $this->name.'/'.$c_name.'.'.$type;
                    $result['thumb'] = $this->name.'/'.$c_name_thumb.'.'.$type;
                    $result['width'] = $imagesize[0];
                    $result['height'] = $imagesize[1];
                }
                return $result;
            }
        } else {
            return $result;
        }
    }
    public function getHookHash($func = '')
    {
        if (!$func) {
            return '';
        }
        return Tools::substr(md5($func), 0, 10);
    }
    public function getNavigate($id_st_product_comment = 0)
    {
        $navs = array();
        $navs[] = array(
            'label' => $this->l('Home'),
            'link' => AdminController::$currentIndex.'&configure='.$this->name.'&token='.Tools::getAdminTokenLite('AdminModules'),
        );
        $parents = StProductCommentClass::getParents($id_st_product_comment);
        if (is_array($parents) && count($parents)) {
            $parents = array_reverse($parents);
            $count = count($parents);
            foreach ($parents as $i => $value) {
                if ($i < $count-1) {
                    $navs[] = array(
                        'label' => $value['customer_name'],
                        'link' => AdminController::$currentIndex.'&configure='.$this->name.'&token='.Tools::getAdminTokenLite('AdminModules').'&view'.$this->name.'&id_st_product_comment='.$value['id_st_product_comment'],
                    );
                } else {
                    $navs[] = array(
                        'label' => $value['customer_name'],
                        'link' => '',
                    );
                }
            }
        }
        $this->smarty->assign('navigate', $navs);
        return $this->display(__FILE__, 'bo_navigation.tpl');
    }
    public function redirectWithNotification($notification, $url)
    {
        if ($notification) {
            $this->context->cookie->notification = $notification;
            $this->context->cookie->write();
        }
        return call_user_func_array(array('Tools', 'redirectAdmin'), array($url));
    }
    protected function prepareNotification()
    {
        $notification = '';
        if (isset($this->context->cookie->notification)) {
            $notification = $this->context->cookie->notification;
            $this->context->cookie->notification = '';
        }
        return $notification;
    }
    public function getConfigError()
    {
        if (!$conf = Tools::getValue('st_conf')) {
            return;
        }
        $errors = array(
            1 => $this->l('An error occurred during deleting.'),
            2 => $this->l('An error occurred during deleting section.'),
            3 => $this->l('An error occurred during creation.'),
            4 => $this->l('An error occurred during updating.'),
            5 => $this->l('An error occurred while updating the status.'),
            6 => $this->l('The settings updated failed.'),
            7 => $this->l('Invalid value for fields.'),
        );
        if (!key_exists($conf, $errors)) {
            return;
        }
        return $this->displayError($errors[$conf]);
    }
    public static function isColor($color)
    {
        return preg_match('/^#[0-9a-fA-F]{6}$/', $color);
    }
    public function sendReminder($cron = true)
    {
        if (!Configuration::get($this->_prefix_st.'ENABLE_REMINDER')) {
            return false;
        }
        $states = array();
        $this->initOrderStates();
        foreach($this->order_states AS $value) {
            if(Configuration::get($this->_prefix_st.strtoupper('order_state_'.$value['id']))) {
                $states[] = $value['id'];
            }
        }
        if (!$states) {
            return false;
        }
        if (!$id_shop = Tools::getValue('id_shop')) {
            $id_shop = (int)$this->context->shop->id;
        }
        if (!$id_lang = Tools::getValue('id_lang')) {
            $id_lang = (int)$this->context->language->id;
        }
        $delay = (int)Configuration::get($this->_prefix_st.'DELAY_REMINDER');
        $max_count = (int)Configuration::get($this->_prefix_st.'REMINDER_TIMES');
        $exclude = Configuration::get($this->_prefix_st.'ID_CATEGORY_EXCLUDED', false);
        $orders = StProductCommentClass::getReminderOrders($states, $delay, $max_count, $id_shop, $exclude);
        foreach($orders as $order) {
            if ($this->sendReminderEmail($order, $id_shop, $id_lang)) {
                StProductCommentClass::updateReminder($order['id_order'], $id_shop);
            }
        }
        return true;
    }
    public function sendReminderEmail($order, $id_shop = 0, $id_lang = 0)
    {
        if (isset($order['id_order']) && $order['id_order'] && isset($order['email']) && $order['email']) {
            $subject = Configuration::get($this->_prefix_st.'EMAIL_REMINDER_SUBJECT', $id_lang);
            $products = '';
            foreach(OrderDetail::getList($order['id_order']) as $val) {
                $products .= '<a href="'.$this->context->link->getProductLink($val['product_id'], null, null, null, $id_lang, $id_shop, $val['product_attribute_id']).'">'.$val['product_name'].'</a><br>';
            }
            $params = array(
                '{name}' => $order['firstname'],
                '{id_order}' => $order['id_order'],
                '{products}' => $products,
            );
            $template = 'reminder';
            // If enable the voucher code, attach it.
            if (Configuration::get($this->_prefix_st.'VOUCHER')) {
                if ($voucher_code = Configuration::get($this->_prefix_st.'VOUCHER_CODE')) {
                    $voucher = CartRule::getCartsRuleByCode($voucher_code, $id_lang);
                    $voucher = array_pop($voucher);
                    if ($voucher['reduction_percent'] > 0) {
                        $params['{voucher_comment}'] = $voucher['reduction_percent'].'%';
                    } else {
                        $params['{voucher_comment}'] = Tools::displayPrice($voucher['reduction_amount'], (int)$voucher['reduction_currency']);
                    }
                    $params['{valid_date}'] = $voucher['date_to'];
                    $template = 'reminder_voucher';
                }
            }
            // Send email to customer.
            if ($this->sendMail($order['email'], $params, $subject, $template, $id_shop, $id_lang)) {
                // Send a copy to webmaster
                if (Configuration::get($this->_prefix_st.'ENABLE_REMINDER_COPY')) {
                    $email = Configuration::get($this->_prefix_st.'SHOP_EMAIL');
                    $subject = $this->l('A new reminder send to customer.');
                    $this->sendMail($email, $params, $subject, 'reminder_admin', $id_shop, $id_lang);
                }
                return true;
            } else {
                echo sprintf($this->l('%sSent mail to [%s] failed, please check your mail server and the mail template whether translated properly.%s'), '<div>', $order['email'], '</div>');
            }
        }
        return false;
    }
    public function sendMail(
        $email, 
        $params, 
        $subject, 
        $template, 
        $id_shop = null, 
        $id_lang = null
    ) {
        if (!$email || !$params || !$subject || !$template) {
            return false;
        }
        if (!$id_shop) {
            $id_shop = (int)$this->context->shop->id;
        }
        if (!$id_lang) {
            $id_lang = (int)$this->context->language->id;
        }
        return Mail::Send(
            $id_lang,
            $template,
            $subject,
            $params,
            $email,
            null,
            null,
            null,
            null,
            null,
            dirname(__FILE__).'/mails/',
            false,
            $id_shop
        );
    }
    
    public function hookActionDeleteGDPRCustomer($customer)
    {
        if (!empty($customer['id_customer'])) {
            $sql = "SELECT id_st_product_comment FROM "._DB_PREFIX_."st_product_comment WHERE id_customer = '".(int)$customer['id_customer']."'";
            if ($rs = Db::getInstance()->executeS($sql)) {
                foreach($rs AS $val) {
                    $object = new StProductCommentClass($val['id_st_product_comment']);
                    $object->delete();
                }
            }
            return json_encode(true);
        }
    }
    public function hookActionExportGDPRData($customer)
    {
        if (!Tools::isEmpty($customer['id_customer'])) {
            $sql = "SELECT * FROM "._DB_PREFIX_."st_product_comment WHERE id_customer = '".(int)$customer['id_customer']."'";
            if ($res = Db::getInstance()->ExecuteS($sql)) {
                return json_encode($res);
            }
            return json_encode($this->l('Product reviews: No data found.'));
       }
   }
}
